/* BdfServer - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.SpecialIncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import java.io.IOException;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.xml.XsltWriter;


/**
 *
 * @author Vincent Calame
 */
public class OdtFicheXsltWriter extends XsltWriter {

    public OdtFicheXsltWriter() {
    }

    public void add(BdfServer bdfServer, Corpus corpus, boolean withCorpusCondition) throws IOException {
        if (corpus == null) {
            openMatchXslTemplate("fiche", null);
            closeXslTemplate();
            return;
        }
        if (withCorpusCondition) {
            openMatchXslTemplate("fiche[@corpus='" + corpus.getSubsetName() + "']", null);
        } else {
            openMatchXslTemplate("fiche", null);
        }
        boolean uniqueLang = BdfTransformationUtils.isUniquelang(bdfServer);
        appendUiComponents(corpus, bdfServer.getUiManager().getMainUiComponents(corpus), uniqueLang);
        closeXslTemplate();
    }

    private void appendUiComponents(Corpus corpus, UiComponents uiComponents, boolean uniqueLang) throws IOException {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        boolean withSoustitre = uiComponents.contains(FieldKey.SOUSTITRE);
        apply(".", "_Title");
        if (withSoustitre) {
            apply("soustitre", "_Inline");
        }
        int p = 0;
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (uiComponent instanceof FieldUi) {
                FieldUi fieldUi = (FieldUi) uiComponent;
                FieldKey fieldKey = fieldUi.getFieldKey();
                if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
                    continue;
                } else if (fieldKey.equals(FieldKey.LANG)) {
                    if (!uniqueLang) {
                        addFieldUi(fieldUi, corpusMetadata);
                    } else {
                        continue;
                    }
                } else {
                    addFieldUi(fieldUi, corpusMetadata);
                }
            } else if (uiComponent instanceof SpecialIncludeUi) {
                addSpecialIncludeUi((SpecialIncludeUi) uiComponent);
            } else if (uiComponent instanceof SubsetIncludeUi) {
                addSubsetIncludeUi((SubsetIncludeUi) uiComponent);
            } else if (uiComponent instanceof CommentUi) {
            }
        }
        apply(".", "_Chrono");
    }

    private void apply(String match, String mode) throws IOException {
        appendIndent();
        addXslApplyTemplates(match, mode);
    }

    private void addFieldUi(FieldUi fieldUi, CorpusMetadata corpusMetadata) throws IOException {
        FieldKey fieldKey = fieldUi.getFieldKey();
        CorpusField corpusField = corpusMetadata.getCorpusField(fieldKey);
        short category = fieldKey.getCategory();
        switch (category) {
            case FieldKey.PROPRIETE_CATEGORY:
                apply("propriete[@name='" + fieldKey.getFieldName() + "']", "_Inline");
                break;
            case FieldKey.SECTION_CATEGORY:
                apply("section[@name='" + fieldKey.getFieldName() + "']", "_Section");
                break;
            case FieldKey.INFORMATION_CATEGORY:
                apply("information[@name='" + fieldKey.getFieldName() + "']", getMode(corpusField));
                break;
            case FieldKey.SPECIAL_CATEGORY:
                if (fieldKey.equals(FieldKey.REDACTEURS)) {
                    apply("redacteurs", "_Inline");
                } else if (fieldKey.equals(FieldKey.LANG)) {
                    apply("lang", "_Inline");
                }
                break;
        }
    }

    private void addSpecialIncludeUi(SpecialIncludeUi includeUi) throws IOException {
        String includeName = includeUi.getName();
        if (includeName.equals(FichothequeConstants.LIAGE_NAME)) {
            apply("extraitcorpus[@extrait-name='" + includeName + "']", "_Listitem");
        }
    }

    private void addSubsetIncludeUi(SubsetIncludeUi includeUi) throws IOException {
        ExtendedIncludeKey includeKey = includeUi.getExtendedIncludeKey();
        String includeName = includeUi.getName();

        switch (includeKey.getCategory()) {
            case SubsetKey.CATEGORY_CORPUS:
                apply("extraitcorpus[@extrait-name='" + includeName + "']", "_Listitem");
                break;
            case SubsetKey.CATEGORY_THESAURUS:
                apply("extraitthesaurus[@extrait-name='" + includeName + "']", "_Inline");
                break;
            case SubsetKey.CATEGORY_ALBUM:
                apply("extraitalbum[@extrait-name='" + includeName + "']", "_Inline");
                break;
            case SubsetKey.CATEGORY_ADDENDA:
                apply("extraitaddenda[@extrait-name='" + includeName + "']", "_Listitem");
                break;
        }
    }

    private static String getMode(CorpusField corpusField) {
        if ((corpusField != null) && (CorpusMetadataUtils.isBlockDisplayInformationField(corpusField))) {
            switch (corpusField.getFicheItemType()) {
                case CorpusField.ITEM_FIELD:
                case CorpusField.PARA_FIELD:
                case CorpusField.IMAGE_FIELD:
                    return "_Block";
                default:
                    return "_Listitem";
            }
        } else {
            return "_Inline";
        }
    }

}
