/* BdfServer - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.instruction;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.api.instruction.BdfCommand;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import net.fichotheque.Fichotheque;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.CommandMessageBuilder;
import net.mapeadores.util.logging.MessageLog;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractBdfCommand implements BdfCommand, BdfInstructionMethods, BdfInstructionConstants {

    protected final RequestMap requestMap;
    protected final BdfServer bdfServer;
    protected final Fichotheque fichotheque;
    protected EditSession editSession;
    protected BdfUser bdfUser;
    private PermissionSummary permissionSummary;
    private CommandMessageBuilder currentCommandMessageBuilder;
    private CommandMessage currentCommandMessage;
    private BdfCommandResultBuilder currentBdfCommandResultBuilder;

    public AbstractBdfCommand(BdfServer bdfServer, RequestMap requestMap) {
        this.bdfServer = bdfServer;
        this.requestMap = requestMap;
        this.fichotheque = bdfServer.getFichotheque();
    }

    @Override
    public CommandMessage testCommand(BdfUser bdfUser) throws BdfInstructionException {
        this.bdfUser = bdfUser;
        currentCommandMessage = null;
        currentCommandMessageBuilder = null;
        currentBdfCommandResultBuilder = null;
        checkParameters();
        if (currentCommandMessage != null) {
            return currentCommandMessage;
        } else if (currentCommandMessageBuilder != null) {
            return currentCommandMessageBuilder.toCommandMessage();
        } else {
            return null;
        }
    }

    @Override
    public BdfCommandResult doCommand(BdfUser bdfUser) throws BdfInstructionException {
        this.bdfUser = bdfUser;
        currentCommandMessage = null;
        currentCommandMessageBuilder = null;
        currentBdfCommandResultBuilder = new BdfCommandResultBuilder();
        if (checkParameters()) {
            doCommand();
            closeEditSession();
        }
        if (currentCommandMessage != null) {
            currentBdfCommandResultBuilder.setCommandMessage(currentCommandMessage);
        } else if (currentCommandMessageBuilder != null) {
            currentBdfCommandResultBuilder.setCommandMessage(currentCommandMessageBuilder.toCommandMessage());
        }
        return currentBdfCommandResultBuilder.toBdfCommandResult();
    }

    @Override
    public BdfServer getBdfServer() {
        return bdfServer;
    }

    @Override
    public RequestMap getRequestMap() {
        return requestMap;
    }

    @Override
    public BdfUser getBdfUser() {
        return bdfUser;
    }

    @Override
    public Fichotheque getFichotheque() {
        return fichotheque;
    }

    @Override
    public PermissionSummary getPermissionSummary() {
        if (permissionSummary == null) {
            permissionSummary = bdfUser.getPermissionSummary(bdfServer);
        }
        return permissionSummary;
    }

    protected abstract boolean checkParameters() throws BdfInstructionException;

    protected abstract void doCommand() throws BdfInstructionException;

    protected void putResultObject(String name, Object obj) {
        if (currentBdfCommandResultBuilder != null) {
            currentBdfCommandResultBuilder.putResultObject(name, obj);
        }
    }

    protected void setError(String errorKey, Object... messageValues) {
        getCommandMessageBuilder().setError(LocalisationUtils.toMessage(errorKey, messageValues));
    }

    protected void addLog(MessageLog messageLog) {
        getCommandMessageBuilder().addLog(messageLog);
    }

    protected void setDone(String doneMessageKey, Object... messageValues) {
        getCommandMessageBuilder().setDone(doneMessageKey, messageValues);
    }

    protected boolean hasMultiError() {
        if (currentCommandMessageBuilder == null) {
            return false;
        } else {
            return currentCommandMessageBuilder.hasMultiError();
        }
    }

    protected void addMultiError(String errorMessageKey, Object... messageValues) {
        getCommandMessageBuilder().addMultiError(LocalisationUtils.toMessage(errorMessageKey, messageValues));
    }

    protected void addMultiError(Message message) {
        getCommandMessageBuilder().addMultiError(message);
    }

    protected void setMultiErrorTitle(String errorMessageKey, Object... messageValues) {
        getCommandMessageBuilder().setError(errorMessageKey, messageValues);
    }

    protected void setCommandMessage(CommandMessage commandMessage) {
        currentCommandMessage = commandMessage;
    }

    protected void initEditSession(String source) {
        if (bdfUser != null) {
            editSession = bdfServer.initEditSession(bdfUser.newEditOrigin(source));
        } else {
            editSession = bdfServer.initEditSession(BdfUserUtils.newEditOrigin(source));
        }
    }

    protected void closeEditSession() {
        if (editSession != null) {
            bdfServer.closeEditSession(editSession);
            editSession = null;
        }
    }

    private CommandMessageBuilder getCommandMessageBuilder() {
        if (currentCommandMessageBuilder == null) {
            currentCommandMessageBuilder = new CommandMessageBuilder();
        }
        return currentCommandMessageBuilder;
    }

}
