/* BdfServer - Copyright (c) 2020-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.instruction;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.PermissionBdfInstructionException;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.roles.PermissionEntry;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.roles.ListFilterEngine;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import java.util.Collections;
import java.util.List;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Document;
import net.fichotheque.album.Illustration;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;


/**
 *
 * @author Vincent Calame
 */
public class PermissionChecker implements ProducerParameters {

    private final BdfServer bdfServer;
    private final BdfUser bdfUser;
    private PermissionSummary permissionSummary = null;

    public PermissionChecker(BdfServer bdfServer, BdfUser bdfUser) {
        this.bdfServer = bdfServer;
        this.bdfUser = bdfUser;
    }

    public PermissionChecker checkGlobalAdmin() throws PermissionBdfInstructionException {
        if (!bdfServer.getPermissionManager().isAdmin(bdfUser.getRedacteur())) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.admin");
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
        return this;
    }

    public PermissionChecker checkAdmin(String... actionNames) throws PermissionBdfInstructionException {
        initPermissionSummary();
        if (actionNames != null) {
            for (String actionName : actionNames) {
                checkAdmin(permissionSummary, actionName);
            }
        }
        return this;
    }

    public PermissionChecker checkHistory(Subset subset, int id) throws PermissionBdfInstructionException {
        SubsetItem subsetItem = subset.getSubsetItemById(id);
        if ((subsetItem != null) && (subsetItem instanceof FicheMeta)) {
            checkRead((FicheMeta) subsetItem);
        } else {
            checkSubsetAdmin(subset);
        }
        return this;
    }

    public PermissionChecker checkSubsetAdmin(Subset subset) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkSubsetAdmin(permissionSummary, subset);
        return this;
    }

    public PermissionChecker checkSubsetAccess(Subset subset) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkSubsetAccess(permissionSummary, subset);
        return this;
    }

    public PermissionChecker checkWrite(Illustration illustration) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkWrite(permissionSummary, illustration);
        return this;
    }

    public PermissionChecker checkWrite(Document document) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkWrite(permissionSummary, document);
        return this;
    }

    public PermissionChecker checkWrite(FicheMeta ficheMeta) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkWrite(permissionSummary, ficheMeta);
        return this;
    }

    public PermissionChecker checkRead(FicheMeta ficheMeta) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkRead(permissionSummary, ficheMeta);
        return this;
    }

    public PermissionChecker checkFicheCreate(Corpus corpus) throws PermissionBdfInstructionException {
        initPermissionSummary();
        checkFicheCreate(permissionSummary, corpus);
        return this;
    }

    @Override
    public BdfServer getBdfServer() {
        return bdfServer;
    }

    @Override
    public BdfUser getBdfUser() {
        return bdfUser;
    }

    @Override
    public PermissionSummary getPermissionSummary() {
        initPermissionSummary();
        return permissionSummary;
    }


    private void initPermissionSummary() {
        if (permissionSummary == null) {
            permissionSummary = PermissionSummaryBuilder.build(bdfServer, bdfUser);
        }
    }

    public static PermissionChecker init(BdfServer bdfServer, BdfUser bdfUser) {
        return new PermissionChecker(bdfServer, bdfUser);
    }

    public static void checkAdmin(PermissionSummary permissionSummary, String actionName) throws PermissionBdfInstructionException {
        if (!permissionSummary.canDo(actionName)) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.admin");
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkSubsetAdmin(PermissionSummary permissionSummary, Subset subset) throws PermissionBdfInstructionException {
        if (!permissionSummary.isSubsetAdmin(subset.getSubsetKey())) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.admin");
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkSubsetAccess(PermissionSummary permissionSummary, Subset subset) throws PermissionBdfInstructionException {
        if (!permissionSummary.hasAccess(subset)) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.read", subset.getSubsetKey().toString());
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkWrite(PermissionSummary permissionSummary, Illustration illustration) throws PermissionBdfInstructionException {
        List<PermissionEntry> list = ListFilterEngine.filter(illustration.getAlbum(), Collections.singletonList((SubsetItem) illustration), permissionSummary);
        if ((list.isEmpty()) || (!list.get(0).isEditable())) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.write", illustration.getAlbum().getSubsetKey().toString() + "/" + illustration.getId());
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkWrite(PermissionSummary permissionSummary, Document document) throws PermissionBdfInstructionException {
        List<PermissionEntry> list = ListFilterEngine.filter(document.getAddenda(), Collections.singletonList((SubsetItem) document), permissionSummary);
        if ((list.isEmpty()) || (!list.get(0).isEditable())) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.write", document.getAddenda().getSubsetKey().toString() + "/" + document.getId());
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkFicheCreate(PermissionSummary permissionSummary, Corpus corpus) throws PermissionBdfInstructionException {
        SubsetKey subsetKey = corpus.getSubsetKey();
        if (!permissionSummary.canCreate(subsetKey)) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.createfiche", subsetKey.toString());
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkWrite(PermissionSummary permissionSummary, FicheMeta ficheMeta) throws PermissionBdfInstructionException {
        if (!permissionSummary.canWrite(ficheMeta)) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.write", FichothequeUtils.toGlobalId(ficheMeta));
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

    public static void checkRead(PermissionSummary permissionSummary, FicheMeta ficheMeta) throws PermissionBdfInstructionException {
        if (!permissionSummary.canRead(ficheMeta)) {
            CommandMessage commandMessage = LogUtils.error("_ error.permission.readfiche", FichothequeUtils.toGlobalId(ficheMeta));
            PermissionBdfInstructionException permissionException = new PermissionBdfInstructionException(commandMessage);
            throw permissionException;
        }
    }

}
