/* BdfServer_Commands - Copyright (c) 2014-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.album.Album;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.FieldOptionsConstants;
import net.fichotheque.include.IncludeKey;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.tools.conversion.imagetoillustration.ImageToIllustrationConverter;
import net.fichotheque.utils.pointeurs.PointeurFactory;
import net.mapeadores.util.logging.MessageLogBuilder;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class ImageToIllustrationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "ImageToIllustration";
    public final static String COMMANDKEY = "_ CRP-26";
    public final static String FIELD_PARAMNAME = "field";
    public final static String ALBUMINCLUDE_PARAMNAME = "albuminclude";
    public final static String BASEURL_PARAMNAME = "baseurl";
    public final static String MODE_PARAMNAME = "mode";
    public final static String RUN_MODE_PARAMVALUE = "run";
    public final static String TEST_MODE_PARAMVALUE = "test";
    private Corpus corpus;
    private Album album;
    private CorpusField sourceField;
    private IncludeKey albumIncludeKey;
    private String baseUrl;
    private boolean testOnly;

    public ImageToIllustrationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        MessageLogBuilder builder = new MessageLogBuilder();
        ImageToIllustrationConverter imageToIllustrationConverter = new ImageToIllustrationConverter(fichothequeEditor.getAlbumEditor(album.getSubsetKey()), fichothequeEditor.getCroisementEditor(), builder, testOnly);
        FichePointeur fichePointeur = PointeurFactory.newFichePointeur(corpus);
        for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
            fichePointeur.setCurrentSubsetItem(ficheMeta);
            imageToIllustrationConverter.convert(fichePointeur, sourceField.getFieldKey(), baseUrl, albumIncludeKey.getMode(), albumIncludeKey.getPoidsFilter());
        }
        addLog(builder.toMessageLog());
        if (testOnly) {
            setDone("_ done.corpus.imagetoillustration_test");
        } else {
            setDone("_ done.corpus.imagetoillustration_run");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdminPermission();
        corpus = getMandatoryCorpus();
        baseUrl = getMandatory(BASEURL_PARAMNAME).trim();
        String fieldParamName = getMandatory(FIELD_PARAMNAME);
        try {
            FieldKey fieldKey = FieldKey.parse(fieldParamName);
            sourceField = corpus.getCorpusMetadata().getCorpusField(fieldKey);
            if (sourceField == null) {
                throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(FIELD_PARAMNAME, fieldParamName));
            }
        } catch (ParseException pe) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(FIELD_PARAMNAME, fieldParamName));
        }
        switch (sourceField.getFicheItemType()) {
            case CorpusField.ITEM_FIELD:
            case CorpusField.IMAGE_FIELD:
                break;
            default:
                setError("_ error.unsupported.ficheitemtype", fieldParamName);
                return false;
        }
        if ((baseUrl.isEmpty()) && (sourceField.getFicheItemType() == CorpusField.IMAGE_FIELD)) {
            String value = sourceField.getStringOption(FieldOptionsConstants.BASEURL_OPTION);
            if (value != null) {
                baseUrl = value;
            }
        }
        String modeString = getMandatory(MODE_PARAMNAME);
        if (modeString.equals(RUN_MODE_PARAMVALUE)) {
            testOnly = false;
        } else {
            testOnly = true;
        }
        String albumIncludeKeyString = getMandatory(ALBUMINCLUDE_PARAMNAME);
        try {
            albumIncludeKey = IncludeKey.parse(albumIncludeKeyString);
        } catch (ParseException pe) {
            setError("_ error.wrong.includekey", albumIncludeKeyString);
            return false;
        }
        if (!albumIncludeKey.getSubsetKey().isAlbumSubset()) {
            setError("_ error.unsupported.subsetcategory", albumIncludeKeyString);
            return false;
        }
        album = (Album) fichotheque.getSubset(albumIncludeKey.getSubsetKey());
        if (album == null) {
            setError("_ error.unknown.album", albumIncludeKeyString);
            return false;
        }
        return true;
    }

}
