/* BdfServer_Commands - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import static fr.exemole.bdfserver.api.instruction.BdfInstructionConstants.CORPUS_OBJ;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.ui.components.CommentUiBuilder;
import fr.exemole.bdfserver.tools.ui.components.FieldUiBuilder;
import fr.exemole.bdfserver.tools.ui.components.IncludeUiBuilder;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.tools.parsers.AttributeParser;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class UiComponentAttributesCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "UiComponentAttributes";
    public final static String COMMANDKEY = "_ CRP-30";
    public final static String ATTRIBUTES_PARAMSUFFIX = "|attributes";
    private Corpus corpus;
    private UiComponents uiComponents;
    private List<UiComponent> uiComponentList;

    public UiComponentAttributesCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        boolean done = false;
        for (UiComponent componentUi : uiComponentList) {
            boolean stepDone = bdfServerEditor.putComponentUi(uiComponents, componentUi);
            if (stepDone) {
                done = true;
            }
        }
        if (done) {
            bdfServerEditor.saveChanges();
            setDone("_ done.global.attributechange");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = getMandatoryCorpus();
        checkSubsetAdminPermission(corpus);
        uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        uiComponentList = new ArrayList<UiComponent>();
        Set<String> existingSet = new HashSet<String>();
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            String name = uiComponent.getName();
            if (!existingSet.contains(name)) {
                existingSet.add(name);
                String attributes = requestMap.getParameter(name + ATTRIBUTES_PARAMSUFFIX);
                if (attributes != null) {
                    AttributeChange attributeChange = AttributeParser.parse(attributes);
                    UiComponent newUiComponent = getNewComponent(uiComponent, attributeChange);
                    if (newUiComponent != null) {
                        uiComponentList.add(newUiComponent);
                    }
                }
            }
        }
        return true;
    }

    private UiComponent getNewComponent(UiComponent uiComponent, AttributeChange attributeChange) {
        if (uiComponent instanceof FieldUi) {
            return FieldUiBuilder.derive((FieldUi) uiComponent)
                    .changesAttribute(attributeChange)
                    .toFieldUi();
        } else if (uiComponent instanceof IncludeUi) {
            return IncludeUiBuilder.derive((IncludeUi) uiComponent)
                    .changesAttribute(attributeChange)
                    .toIncludeUi();
        } else if (uiComponent instanceof CommentUi) {
            return CommentUiBuilder.derive(bdfServer.getUiManager().getTrustedHtmlFactory(), (CommentUi) uiComponent)
                    .changesAttribute(attributeChange)
                    .toCommentUi();
        } else {
            return null;
        }
    }

}
