/* BdfServer_Commands - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.thesaurus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.users.BdfUserConstants;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.ParentRecursivityException;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.thesaurus.ThesaurusTools;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotcleMergeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "MotcleMerge";
    public final static String COMMANDKEY = "_ THS-18";
    public static String MERGETHESAURUS_PARAMNAME = "mergethesaurus";
    public static String MERGEMOTCLE_PARAMNAME = "mergemotcle";
    private Motcle motcle;
    private Motcle mergeMotcle;

    public MotcleMergeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.THESAURUS + "/" + COMMANDNAME);
        Thesaurus mergeThesaurus = mergeMotcle.getThesaurus();
        Lang workingLang = bdfUser.getWorkingLang();
        Object[] objs = new Object[2];
        objs[0] = getCompleteLabelString(motcle, workingLang);
        objs[1] = getCompleteLabelString(mergeMotcle, workingLang);
        try {
            Langs langs = bdfServer.getThesaurusLangChecker().getAuthorizedLangs(mergeThesaurus);
            ThesaurusTools.merge(editSession.getFichothequeEditor(), motcle, mergeMotcle, langs);
        } catch (ParentRecursivityException pre) {
            throw new ShouldNotOccurException("test done before");
        }
        String messageKey = (mergeThesaurus.equals(motcle.getThesaurus())) ? "_ done.thesaurus.motclemerge_samethesaurus" : "_ done.thesaurus.motclemerge_otherthesaurus";
        setDone(messageKey, objs);
        bdfUser.putParameter(BdfUserConstants.THESAURUSMERGELAST_SUBSETKEY_OBJ, mergeThesaurus.getSubsetKey());
        putResultObject(THESAURUS_OBJ, mergeThesaurus);
        putResultObject(MOTCLE_OBJ, mergeMotcle);
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        motcle = getMandatoryMotcle();
        String mergeThesaurusName = getMandatory(MERGETHESAURUS_PARAMNAME);
        Thesaurus mergeThesaurus = FichothequeUtils.getThesaurus(fichotheque, mergeThesaurusName);
        if (mergeThesaurus == null) {
            setError("_ error.unknown.thesaurus", mergeThesaurusName);
            return false;
        }
        PermissionChecker.init(bdfServer, bdfUser)
                .checkSubsetAdmin(motcle.getThesaurus())
                .checkSubsetAccess(mergeThesaurus);
        if (!checkConfirmation()) {
            setError("_ error.empty.confirmationcheck_remove");
            return false;
        }
        String idMergeString = getMandatory(MERGEMOTCLE_PARAMNAME);
        try {
            mergeMotcle = BdfInstructionUtils.getMotcle(mergeThesaurus, idMergeString);
        } catch (NumberFormatException nfe) {
        }
        if (mergeMotcle == null) {
            setError("_ error.unknown.mergemotcle", idMergeString);
            return false;
        }
        if (ThesaurusUtils.isDescendant(mergeMotcle, motcle)) {
            setError("_ error.unsupported.recursivemerge", idMergeString);
            return false;
        }
        return true;
    }

    private String getCompleteLabelString(Motcle motcle, Lang lang) {
        StringBuilder buf = new StringBuilder();
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            buf.append(idalpha);
            buf.append(" \u2013 ");
        }
        String labelString;
        if (ThesaurusUtils.isBabelienType(motcle.getThesaurus())) {
            labelString = ThesaurusUtils.getBabelienLabel(motcle).getLabelString();
        } else {
            Label label = LabelUtils.getLangPartCheckedLabel(motcle.getLabels(), lang);
            if (label != null) {
                labelString = label.getLabelString();
            } else {
                labelString = "?";
            }
        }
        buf.append(labelString);
        if (idalpha == null) {
            buf.append(" (id=");
            buf.append(String.valueOf(motcle.getId()));
            buf.append(")");
        }
        return buf.toString();
    }

}
