/* BdfServer_Get - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstruction;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.get.streamproducers.ExtractionStream;
import fr.exemole.bdfserver.get.streamproducers.MissingFicheHtmlProducer;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.text.ParseException;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.utils.CorpusUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.ServletUtils;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.text.FileName;
import net.mapeadores.util.text.ValidExtension;


/**
 *
 * @author Vincent Calame
 */
public class FichesInstruction implements BdfInstruction {

    private final BdfServer bdfServer;
    private final RequestMap requestMap;
    private final String filePath;
    private Corpus corpus;
    private FicheMeta ficheMeta;

    public FichesInstruction(BdfServer bdfServer, RequestMap requestMap, String filePath) {
        this.bdfServer = bdfServer;
        this.filePath = filePath;
        this.requestMap = requestMap;
    }

    @Override
    public short getBdfUserNeed() {
        return BdfInstructionConstants.MANDATORY_BDFUSER;
    }

    @Override
    public ResponseHandler runInstruction(BdfUser bdfUser) {
        FileName fileName;
        try {
            fileName = FileName.parse(filePath);
        } catch (ParseException pe) {
            return null;
        }
        String baseName = fileName.getBasename();
        ValidExtension validExtension;
        try {
            validExtension = ValidExtension.parse(fileName.getExtension());
        } catch (ParseException pe) {
            throw new HttpAccessException("_ error.wrong.extension", fileName.getExtension());
        }
        String[] tokens = baseName.split("-");
        int length = tokens.length;
        if (length < 1) {
            return null;
        }
        if (tokens[0].equals(BdfInstructionUtils.COMPILATION_TOKEN)) {
            String templateName = "";
            if (length > 1) {
                templateName = tokens[1];
            }
            return getCompilationResponseHandler(bdfUser, validExtension, templateName);
        }
        if (length < 2) {
            return null;
        }
        try {
            SubsetKey corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, tokens[0]);
            corpus = (Corpus) bdfServer.getFichotheque().getSubset(corpusKey);
        } catch (java.text.ParseException pe) {
            throw new HttpAccessException("_ error.wrong.corpusname", tokens[0]);
        }
        if (corpus == null) {
            throw new HttpAccessException("_ error.unknown.corpus", tokens[0]);
        }
        String idString = tokens[1];
        if (idString.equals("test")) {
            if (corpus.size() == 0) {
                throw new HttpAccessException("_ error.unsupported.emptycorpus");
            }
            ficheMeta = getTestFicheMeta(corpus, bdfUser);
        } else {
            try {
                int id = Integer.parseInt(idString);
                if (id < 1) {
                    throw new HttpAccessException("_ error.wrong.id", idString);
                }
                ficheMeta = corpus.getFicheMetaById(id);
                if (ficheMeta == null) {
                    return getMissingFicheMetaResponseHandler(bdfUser, corpus, id, validExtension);
                }
            } catch (NumberFormatException nfe) {
                throw new HttpAccessException("_ error.wrong.id", idString);
            }
        }
        String templateName = "";
        if (length > 2) {
            templateName = tokens[2];
            if (templateName.isEmpty()) {
                templateName = TemplateKey.DEFAULT_NAME;
            }
        }
        return getFicheResponseHandler(bdfUser, ficheMeta, validExtension, templateName);
    }

    private FicheMeta getTestFicheMeta(Corpus corpus, BdfUser bdfUser) {
        List<FicheMeta> ficheMetaList = CorpusUtils.getFicheMetaListByCorpus(bdfUser.getSelectedFiches(), corpus);
        if (!ficheMetaList.isEmpty()) {
            return ficheMetaList.get(0);
        }
        return corpus.getFicheMetaList().get(0);
    }

    private ResponseHandler getFicheResponseHandler(BdfUser bdfUser, FicheMeta ficheMeta, ValidExtension validExtension, String templateName) {
        PermissionSummary permissionSummary = bdfUser.getPermissionSummary(bdfServer);
        boolean canRead = permissionSummary.canRead(ficheMeta);
        if (!canRead) {
            if (validExtension.toString().equals("html")) {
                MessageLocalisation messageLocalisation = BdfUserUtils.getMessageLocalisation(bdfServer, bdfUser);
                return CommandMessageResponseHandler.getHtmlInstance(messageLocalisation, LogUtils.error("_ error.permission.readfiche", FichothequeUtils.toGlobalId(ficheMeta)));
            } else {
                throw new HttpAccessException(HttpAccessException.FORBIDDEN_ACCESS);
            }
        }
        TransformationKey transformationKey = new TransformationKey(ficheMeta.getCorpus().getSubsetKey());
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        switch (validExtension.toString()) {
            case "xml":
                return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, extractionContext, transformationKey, templateName, ficheMeta);
            case "html":
                return ExtractionStream.toSimpleTemplateResponseHandler(bdfServer, bdfUser, extractionContext, transformationKey, templateName, ficheMeta);
            default:
                return ExtractionStream.toStreamTemplateResponseHandler(bdfServer, bdfUser, extractionContext, transformationKey, validExtension, templateName, ficheMeta);
        }
    }

    private ResponseHandler getCompilationResponseHandler(BdfUser bdfUser, ValidExtension validExtension, String templateName) {
        int limit = 0;
        String limitString = requestMap.getParameter("limit");
        if (limitString != null) {
            try {
                limit = Integer.parseInt(limitString);
            } catch (NumberFormatException nfe) {

            }
        }
        TransformationKey transformationKey = TransformationKey.COMPILATION_TRANSFORMATIONKEY;
        Fiches fiches = bdfUser.getSelectedFiches();
        if (limit > 0) {
            fiches = CorpusUtils.reduce(fiches, limit);
        }
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(bdfUser.getPermissionSummary(bdfServer).getSubsetAccessPredicate())
                .setSelectedFiches(fiches)
                .toExtractionContext();
        switch (validExtension.toString()) {
            case "xml":
                return ExtractionStream.toXmlResponseHandler(bdfServer, bdfUser, extractionContext, transformationKey, templateName, null);
            case "html":
                return ExtractionStream.toSimpleTemplateResponseHandler(bdfServer, bdfUser, extractionContext, transformationKey, templateName, null);
            default:
                return ExtractionStream.toStreamTemplateResponseHandler(bdfServer, bdfUser, extractionContext, transformationKey, validExtension, templateName, null);
        }
    }

    private ResponseHandler getMissingFicheMetaResponseHandler(BdfUser bdfUser, Corpus corpus, int ficheId, ValidExtension validExtension) {
        if (!validExtension.toString().equals("html")) {
            throw new HttpAccessException("_ error.unknown.id", String.valueOf(ficheId));
        }
        PermissionSummary permissionSummary = PermissionSummaryBuilder.build(bdfServer, bdfUser);
        SubsetKey corpusKey = corpus.getSubsetKey();
        int level = permissionSummary.getReadLevel(corpusKey);
        if (level == PermissionSummary.SUMMARYLEVEL_0_NONE) {
            MessageLocalisation messageLocalisation = BdfUserUtils.getMessageLocalisation(bdfServer, bdfUser);
            return CommandMessageResponseHandler.getHtmlInstance(messageLocalisation, LogUtils.error("_ error.permission.readfiche", FichothequeUtils.toGlobalId(corpusKey, ficheId)));
        }
        MissingFicheHtmlProducer missingFicheHtmlProducer = new MissingFicheHtmlProducer(bdfServer, bdfUser, permissionSummary, corpus, ficheId);
        return ServletUtils.wrap(missingFicheHtmlProducer);
    }

}
