/* BdfServer_Html - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.ficheform.AddendaIncludeElement;
import fr.exemole.bdfserver.api.ficheform.AlbumIncludeElement;
import fr.exemole.bdfserver.api.ficheform.CorpusIncludeElement;
import fr.exemole.bdfserver.api.ficheform.FormElement;
import fr.exemole.bdfserver.api.ficheform.HiddenIncludeElement;
import fr.exemole.bdfserver.api.ficheform.RattachementElement;
import fr.exemole.bdfserver.api.ficheform.ThesaurusIncludeElement;
import fr.exemole.bdfserver.api.interaction.domains.PiocheDomain;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.json.ThesaurusSuggestionJson;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.ui.MetadataPhraseDefCatalog;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import net.fichotheque.SubsetItem;
import net.fichotheque.addenda.Document;
import net.fichotheque.addenda.Version;
import net.fichotheque.album.Illustration;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.AlbumUtils;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlConstants;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlWrapper;
import net.mapeadores.util.html.WrapperFactory;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.UserLangContext;


/**
 *
 * @author Vincent Calame
 */
public final class IncludeFormHtml {

    private final static HtmlWrapper DOCUMENT_LIST_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "document-list").classes("ficheform-standard-Cell"));
    private final static HtmlWrapper DOCUMENT_INPUT_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "document-input").classes("ficheform-standard-Cell"));
    private final static HtmlWrapper DOCUMENT_FILE_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "document-file").classes("ficheform-standard-Cell ficheform-document-FileCell"));
    private final static HtmlWrapper ILLUSTRATION_LIST_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "illustration-list").classes("ficheform-standard-Cell"));
    private final static HtmlWrapper ILLUSTRATION_INPUT_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "illustration-input").classes("ficheform-standard-Cell"));
    private final static HtmlWrapper ILLUSTRATION_FILE_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "illustration-file").classes("ficheform-standard-Cell ficheform-illustration-FileCell"));
    private final static HtmlWrapper FICHE_LIST_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "fiche-list").classes("ficheform-standard-Cell ficheform-Auto"));
    private final static HtmlWrapper FICHE_INPUT_MEDIUM_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "fiche-input").classes("ficheform-standard-Cell ficheform-Medium"));
    private final static HtmlWrapper FICHE_INPUT_SMALL_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "fiche-input").classes("ficheform-standard-Cell ficheform-Small"));
    private final static HtmlWrapper MOTCLE_INPUT_LARGE_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "motcle-input").classes("ficheform-standard-Cell ficheform-Large"));
    private final static HtmlWrapper MOTCLE_INPUT_MEDIUM_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "motcle-input").classes("ficheform-standard-Cell ficheform-Medium"));
    private final static HtmlWrapper MOTCLE_INPUT_SMALL_CELL = WrapperFactory.div(HA.attr("data-ficheform-role", "motcle-input").classes("ficheform-standard-Cell ficheform-Small"));
    private final static HtmlWrapper LARGE_CELL = WrapperFactory.div("ficheform-standard-Cell ficheform-Large");

    private IncludeFormHtml() {
    }

    public static boolean printAddendaInclude(HtmlPrinter hp, AddendaIncludeElement formElement, String namePrefix, boolean withFileForm, Lang lang) {
        String name = getPrefixedName(formElement, namePrefix);
        String addendaName = formElement.getAddenda().getSubsetName();
        hp
                .SECTION(toEntryAttributes(name, formElement, "addenda-include").attr("data-subset-name", addendaName))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, null))
                .__(DOCUMENT_LIST_CELL, () -> {
                    List<Document> documentList = formElement.getDocumentList();
                    if (!documentList.isEmpty()) {
                        for (Document document : documentList) {
                            int documentId = document.getId();
                            hp
                                    .DIV(HA.id(hp.generateId()).attr("data-ficheform-role", "document").classes("ficheform-document-Block").attr("data-subset-item-id", String.valueOf(documentId)))
                                    .SPAN(HA.attr("data-ficheform-role", "document-uri").classes("ficheform-document-Uri"))
                                    .__escape(FichothequeUtils.toGlobalId(document))
                                    ._SPAN()
                                    .__dash()
                                    .SPAN(HA.attr("data-ficheform-role", "document-basename").classes("ficheform-document-Basename"))
                                    .__escape(document.getBasename())
                                    ._SPAN()
                                    .__space()
                                    .SPAN(HA.attr("data-ficheform-role", "document-extensions").classes("ficheform-document-Extensions"))
                                    .__escape("[ ");
                            boolean next = false;
                            for (Version version : document.getVersionList()) {
                                if (next) {
                                    hp
                                            .__escape(" / ");
                                } else {
                                    next = true;
                                }
                                String extension = version.getExtension();
                                hp
                                        .A(HA.href("documents/" + addendaName + "/" + document.getBasename() + "." + extension).target(HtmlConstants.BLANK_TARGET))
                                        .__escape('.')
                                        .__escape(extension)
                                        ._A();
                            }
                            hp
                                    .__escape(" ]")
                                    ._SPAN()
                                    ._DIV();
                        }
                    }
                })
                .__(DOCUMENT_INPUT_CELL, () -> {
                    hp
                            .INPUT(hp.name(name + ":create").type(HtmlConstants.HIDDEN_TYPE).attr("data-ficheform-role", "createfield"))
                            .INPUT(hp.name(name + ":remove").type(HtmlConstants.HIDDEN_TYPE).attr("data-ficheform-role", "removefield"))
                            .INPUT(hp.name(name + ":change").type(HtmlConstants.HIDDEN_TYPE).attr("data-ficheform-role", "changefield"));
                    if (formElement.isAddAllowed()) {
                        hp
                                .INPUT(hp.name(name + ":add").type(HtmlConstants.HIDDEN_TYPE).attr("data-ficheform-role", "addfield"));
                    }
                });
        if (withFileForm) {
            hp
                    .__(DOCUMENT_FILE_CELL, () -> {
                        HtmlAttributes fileInput = hp.name(name + ":file").type(HtmlConstants.FILE_TYPE).addClass("ficheform-Full global-FileInput").size("70").multiple(true);
                        hp
                                .LABEL_for(fileInput.id())
                                .__localize("_ label.edition.documentfileinput")
                                .__colon()
                                ._LABEL()
                                .INPUT(fileInput);
                    });
        }
        hp
                ._SECTION();
        return true;
    }

    public static boolean printAlbumInclude(HtmlPrinter hp, AlbumIncludeElement formElement, String namePrefix, boolean withUploadForm, Lang lang) {
        String name = getPrefixedName(formElement, namePrefix);
        String albumName = formElement.getAlbum().getSubsetName();
        hp
                .SECTION(toEntryAttributes(name, formElement, "album-include")
                        .attr("data-subset-name", albumName))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, null))
                .__(ILLUSTRATION_LIST_CELL, () -> {
                    List<Illustration> illustrationList = formElement.getIllustrationList();
                    if (!illustrationList.isEmpty()) {
                        for (Illustration illustration : illustrationList) {
                            int illustrationId = illustration.getId();
                            hp
                                    .DIV(HA.id(hp.generateId()).classes("ficheform-illustration-Block").attr("data-ficheform-role", "illustration").attr("data-subset-item-id", String.valueOf(illustrationId)))
                                    .DIV("ficheform-illustration-Image")
                                    .IMG(HA.src("illustrations/_mini/" + AlbumUtils.toFileName(illustration)).attr("data-ficheform-role", "illustration-image"))
                                    ._DIV()
                                    .DIV(HA.attr("data-ficheform-role", "illustration-uri").classes("ficheform-illustration-Uri"))
                                    .__escape(albumName + "/" + illustrationId)
                                    ._DIV()
                                    ._DIV();
                        }
                    }
                })
                .__(ILLUSTRATION_INPUT_CELL, () -> {
                    hp
                            .INPUT(HA.type(HtmlConstants.HIDDEN_TYPE).name(name + ":create").attr("data-ficheform-role", "createfield"))
                            .INPUT(HA.type(HtmlConstants.HIDDEN_TYPE).name(name + ":remove").attr("data-ficheform-role", "removefield"))
                            .INPUT(HA.type(HtmlConstants.HIDDEN_TYPE).name(name + ":update").attr("data-ficheform-role", "updatefield"));
                });
        if (withUploadForm) {
            hp
                    .__(ILLUSTRATION_FILE_CELL, () -> {
                        HtmlAttributes fileInput = hp.name(name + ":file").type(HtmlConstants.FILE_TYPE).addClass("ficheform-Full global-FileInput").size("70").multiple(true).attr("accept", "png,jpg,jpeg,bmp,gif");
                        hp
                                .LABEL_for(fileInput.id())
                                .__localize("_ label.edition.illustrationfileinput")
                                .__colon()
                                ._LABEL()
                                .SPAN("ficheform-illustration-Info")
                                .__localize("_ info.album.illustrationformat")
                                ._SPAN()
                                .INPUT(fileInput);
                    });
        }
        hp
                ._SECTION();
        return true;
    }

    public static boolean printHiddenInclude(HtmlPrinter hp, HiddenIncludeElement formElement, String namePrefix) {
        String name = getPrefixedName(formElement, namePrefix);
        hp
                .INPUT_hidden(name, formElement.getValue());
        return true;
    }

    public static boolean printCorpusInclude(HtmlPrinter hp, CorpusIncludeElement formElement, String namePrefix, UserLangContext userLangContext) {
        String corpusName;
        String entryType;
        Corpus corpus = formElement.getCorpus();
        HtmlWrapper cell;
        if (corpus != null) {
            corpusName = corpus.getSubsetKey().getSubsetName();
            entryType = "corpus-include";
            cell = FICHE_INPUT_SMALL_CELL;
        } else {
            corpusName = null;
            entryType = "liage";
            cell = FICHE_INPUT_MEDIUM_CELL;
        }
        String name = getPrefixedName(formElement, namePrefix);
        String motcleQueryString = formElement.getMotcleQueryString();
        String genId = hp.generateId();
        HtmlAttributes inputAttributes = HA.name(name).id(genId).attr("data-ficheform-role", "textinput").classes("ficheform-Full");
        int rows = formElement.getRows();
        hp
                .SECTION(toEntryAttributes(name, formElement, entryType)
                        .attr("data-ficheform-limit", "-1")
                        .attr((!formElement.hasPoidsFilter()), "data-ficheform-withpoids", "1")
                        .attr((motcleQueryString != null), "data-ficheform-select-options", motcleQueryString)
                        .attr("data-subset-name", corpusName))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, userLangContext.getWorkingLang(), genId))
                .__(printFicheList(hp, formElement, name, userLangContext))
                .__(cell, () -> {
                    if (rows == 1) {
                        hp
                                .INPUT_text(inputAttributes.size("29"));
                    } else {
                        hp
                                .TEXTAREA(inputAttributes.cols(39).rows(rows))
                                ._TEXTAREA();
                    }
                })
                ._SECTION();
        return true;
    }


    public static boolean printTextThesaurusInclude(BdfServer bdfServer, HtmlPrinter hp, ThesaurusIncludeElement.Text formElement, String namePrefix, Lang lang) {
        String name = getPrefixedName(formElement, namePrefix);
        String thesaurusName = formElement.getThesaurus().getSubsetName();
        String widthType = formElement.getWidthType();
        int defaultSize = CommonFormHtml.getDefaultSize(widthType);
        String genId = hp.generateId();
        HtmlAttributes inputAttr = HA.name(name).id(genId).classes("ficheform-Full").attr("data-ficheform-role", "textinput");
        if ((hp.isWithJavascript()) && (formElement.isWithSuggestion())) {
            Lang thesaurusLang = BdfServerUtils.checkLangDisponibility(bdfServer, formElement.getThesaurus(), lang);
            inputAttr.attr("data-ficheform-suggestion", ThesaurusSuggestionJson.toString(formElement.getThesaurus(), thesaurusLang, " – ", formElement.getDestinationSubsetKey()));
        }
        int rows = formElement.getRows();
        hp
                .SECTION(toEntryAttributes(name, formElement, "thesaurus-include")
                        .attr("data-ficheform-limit", "-1")
                        .attr(formElement.isWithExternalSource(), "data-ficheform-externalsource", "1")
                        .attr("data-subset-name", thesaurusName))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, genId))
                .__(motcleInputCell(widthType), () -> {
                    if (rows == 1) {
                        hp
                                .INPUT_text(inputAttr.size(String.valueOf(defaultSize)).value(formElement.getValue()));
                    } else {
                        hp
                                .TEXTAREA(inputAttr.cols(defaultSize).rows(rows))
                                .__escape(formElement.getValue(), true)
                                ._TEXTAREA();
                    }
                })
                ._SECTION();
        return true;
    }

    public static boolean printListThesaurusInclude(HtmlPrinter hp, ThesaurusIncludeElement.List formElement, String namePrefix, Lang lang) {
        String name = getPrefixedName(formElement, namePrefix);
        String thesaurusName = formElement.getThesaurus().getSubsetName();
        String listType = formElement.getListType();
        Set<Motcle> selectedMotcleSet = formElement.getSelectedMotcleSet();
        List<InputItem> inputItemList = new ArrayList<InputItem>();
        if ((formElement.isNoneAllowed()) && (!listType.equals(InputOption.INPUT_CHECK))) {
            inputItemList.add(new InputItem("", "---", selectedMotcleSet.isEmpty(), 1, false));
        } else if (listType.equals(InputOption.INPUT_SELECT)) {
            if ((!formElement.isNewIndexation()) && (selectedMotcleSet.isEmpty())) {
                inputItemList.add(new InputItem("", "---", true, 1, false));
            }
        }
        InputItem.populateInputItems(formElement, lang, inputItemList);
        HtmlAttributes entryAttr = toEntryAttributes(name, formElement, null).attr("data-subset-name", thesaurusName);
        if (listType.equals(InputOption.INPUT_SELECT)) {
            HtmlAttributes selectAttr = hp.name(name);
            hp
                    .SECTION(entryAttr)
                    .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, selectAttr.id()))
                    .__(LARGE_CELL, () -> {
                        hp
                                .SELECT(selectAttr)
                                .__(InputItem.printOptions(hp, inputItemList))
                                ._SELECT();
                    })
                    ._SECTION();
        } else {
            HtmlAttributes attrModel = HA.name(name).attr("data-ficheform-grouped", "1");
            if (listType.equals(InputOption.INPUT_CHECK)) {
                attrModel.type(HtmlConstants.CHECKBOX_TYPE);
            } else {
                attrModel.type(HtmlConstants.RADIO_TYPE);
            }
            hp
                    .SECTION(entryAttr)
                    .__(CommonFormHtml.printStandardLabel(hp, formElement, lang, null))
                    .__(LARGE_CELL, () -> {
                        if (listType.equals(InputOption.INPUT_CHECK)) {
                            hp
                                    .INPUT_hidden(name, "");
                        }
                        hp
                                .__(InputItem.printInputList(hp, inputItemList, attrModel));
                    })
                    ._SECTION();
        }
        return true;
    }

    public static boolean printNotEditableThesaurusInclude(HtmlPrinter hp, ThesaurusIncludeElement.NotEditable formElement, String namePrefix, Lang lang) {
        String name = getPrefixedName(formElement, namePrefix);
        StringBuilder valueBuffer = new StringBuilder();
        StringBuilder textBuffer = new StringBuilder();
        Set<Motcle> selectedMotcleSet = formElement.getSelectedMotcleSet();
        if (!selectedMotcleSet.isEmpty()) {
            boolean premier = true;
            int idalphaStyle = InputItem.getStyleValue(formElement.getIdalphaStyle());
            for (Motcle motcle : selectedMotcleSet) {
                if (premier) {
                    premier = false;
                } else {
                    valueBuffer.append(";");
                    textBuffer.append(" ; ");
                }
                String idalpha = motcle.getIdalpha();
                if (idalpha == null) {
                    valueBuffer.append(motcle.getId());
                } else {
                    valueBuffer.append(idalpha);
                }
                textBuffer.append(InputItem.getText(motcle, lang, idalphaStyle));
            }
        }
        hp
                .SECTION(toEntryAttributes(name, false, formElement.getAttributes(), null))
                .INPUT_hidden(name, valueBuffer.toString())
                .__(printNotEditableLabelSpans(hp, formElement.getLabel()))
                .__(LARGE_CELL, () -> {
                    hp
                            .__escape(textBuffer.toString());
                })
                ._SECTION();
        return true;
    }

    public static boolean printFicheStyleThesaurusInclude(HtmlPrinter hp, ThesaurusIncludeElement.FicheStyle formElement, String namePrefix, UserLangContext userLangContext) {
        Thesaurus thesaurus = formElement.getThesaurus();
        HtmlWrapper cell = MOTCLE_INPUT_SMALL_CELL;
        String name = getPrefixedName(formElement, namePrefix);
        String genId = hp.generateId();
        HtmlAttributes inputAttributes = HA.name(name).id(genId).attr("data-ficheform-role", "textinput").classes("ficheform-Full");
        hp
                .SECTION(toEntryAttributes(name, formElement, "thesaurus-include")
                        .attr("data-ficheform-limit", "-1")
                        .attr(formElement.isWithExternalSource(), "data-ficheform-externalsource", "1")
                        .attr(!formElement.isWithExternalSource(), "data-ficheform-wanted", PiocheDomain.CODE_ID_WANTED)
                        .attr((!formElement.hasPoidsFilter()), "data-ficheform-withpoids", "1")
                        .attr("data-subset-name", thesaurus.getSubsetName()))
                .__(CommonFormHtml.printStandardLabel(hp, formElement, userLangContext.getWorkingLang(), genId))
                .__(printFicheStyleMotcleList(hp, formElement, name, userLangContext))
                .__(cell, () -> {
                    hp
                            .INPUT_text(inputAttributes.size("29"));
                })
                ._SECTION();
        return true;
    }


    public static boolean printRattachement(BdfServer bdfServer, HtmlPrinter hp, RattachementElement formElement, UserLangContext userLangContext) {
        SubsetItem masterSubsetItem = formElement.getMasterSubsetItem();
        if (masterSubsetItem != null) {
            printRattachementToMaster(bdfServer, hp, formElement, userLangContext);
        } else {
            List<FicheMeta> satelliteFicheMetaList = formElement.getSatelliteFicheMetaList();
            hp
                    .SECTION("ficheform-standard-Entry ficheform-NotEditable")
                    .__(printNotEditableLabelSpans(hp, formElement.getLabel()))
                    .__(LARGE_CELL, () -> {
                        if (satelliteFicheMetaList.isEmpty()) {
                            hp
                                    .EM()
                                    .__localize("_ info.edition.satellite_none")
                                    ._EM();
                        } else {
                            boolean premier = true;
                            Lang workingLang = userLangContext.getWorkingLang();
                            Locale formatLocale = userLangContext.getFormatLocale();
                            for (FicheMeta ficheMeta : satelliteFicheMetaList) {
                                if (premier) {
                                    premier = false;
                                } else {
                                    hp
                                            .BR();
                                }
                                hp
                                        .__escape(CorpusMetadataUtils.getFicheTitle(ficheMeta, workingLang, formatLocale));
                            }
                        }
                    })
                    ._SECTION();
        }
        return true;
    }

    private static boolean printRattachementToMaster(BdfServer bdfServer, HtmlPrinter hp, RattachementElement formElement, UserLangContext userLangContext) {
        Lang workingLang = userLangContext.getWorkingLang();
        List<FicheMeta> satelliteFicheMetaList = formElement.getSatelliteFicheMetaList();
        SubsetItem subsetItem = formElement.getMasterSubsetItem();
        hp
                .SECTION("ficheform-standard-Entry ficheform-NotEditable")
                .__(printNotEditableLabelSpans(hp, formElement.getLabel()))
                .__(LARGE_CELL, () -> {
                    Locale formatLocale = userLangContext.getFormatLocale();
                    if (subsetItem instanceof Motcle) {
                        Motcle motcle = (Motcle) subsetItem;
                        hp
                                .__(BdfHtmlUtils.printMotcleLabel(hp, motcle, BdfServerUtils.checkLangDisponibility(bdfServer, motcle.getThesaurus(), workingLang)));
                    } else {
                        hp
                                .__escape(CorpusMetadataUtils.getFicheTitle((FicheMeta) subsetItem, workingLang, formatLocale));
                    }
                    if (!satelliteFicheMetaList.isEmpty()) {
                        hp
                                .UL();
                        for (FicheMeta satelliteFicheMeta : satelliteFicheMetaList) {
                            hp
                                    .LI()
                                    .__escape(CorpusMetadataUtils.getFicheTitle(satelliteFicheMeta, workingLang, formatLocale))
                                    ._LI();
                        }
                        hp
                                ._UL();
                    }
                })
                ._SECTION();
        return true;
    }

    public static boolean printNotEditableLabelSpans(HtmlPrinter hp, String label) {
        hp
                .__breakLine()
                .SPAN("ficheform-standard-Label")
                .__escape(label)
                ._SPAN()
                .__breakLine()
                .SPAN("ficheform-standard-Colon")
                .__escape(':')
                ._SPAN()
                .__breakLine();
        return true;
    }

    private static String getPrefixedName(FormElement.Include includeElement, String namePrefix) {
        return IncludeFormHtml.getPrefixedName(includeElement.getIncludeName(), namePrefix);
    }

    private static String getPrefixedName(String name, String namePrefix) {
        if ((namePrefix != null) && (namePrefix.length() > 0)) {
            name = namePrefix + "_" + name;
        }
        return name;
    }

    private static HtmlAttributes toEntryAttributes(String name, FormElement formElement, String entryType) {
        return toEntryAttributes(name, formElement.isMandatory(), formElement.getAttributes(), entryType);
    }

    private static HtmlAttributes toEntryAttributes(String name, boolean isMandatory, Attributes attributes, String entryType) {
        String key = name.replace(':', '_');
        HtmlAttributes htmlAttributes = HA
                .attr("data-ficheform-role", "entry")
                .attr("data-ficheform-key", key)
                .attr(isMandatory, "data-ficheform-mandatory", "1")
                .attr((entryType != null), "data-ficheform-entry", entryType)
                .classes("ficheform-standard-Entry");
        CommonFormHtml.addClasses(htmlAttributes, attributes);
        return htmlAttributes;
    }

    private static HtmlWrapper motcleInputCell(String widthType) {
        switch (widthType) {
            case InputOption.WIDTH_SMALL:
                return MOTCLE_INPUT_SMALL_CELL;
            case InputOption.WIDTH_MEDIUM:
                return MOTCLE_INPUT_MEDIUM_CELL;
            case InputOption.WIDTH_LARGE:
                return MOTCLE_INPUT_LARGE_CELL;
            default:
                return MOTCLE_INPUT_LARGE_CELL;
        }
    }

    private static boolean printFicheList(HtmlPrinter hp, CorpusIncludeElement formElement, String name, UserLangContext userLangContext) {
        List<CorpusIncludeElement.Entry> entryList = formElement.getEntryList();
        Lang workingLang = userLangContext.getWorkingLang();
        Locale formatLocale = userLangContext.getFormatLocale();
        if (entryList.isEmpty()) {
            return false;
        }
        hp
                .__(FICHE_LIST_CELL, () -> {
                    if (hp.isWithJavascript()) {
                        for (CorpusIncludeElement.Entry entry : entryList) {
                            printFicheListItem(hp, name, entry.getValue(), entry.getPoids(), CorpusMetadataUtils.getFicheTitle(entry.getFicheMeta(), workingLang, formatLocale));
                        }
                    } else {
                        for (CorpusIncludeElement.Entry entry : entryList) {
                            HtmlAttributes input = hp.name(name).value(entry.getValue()).size("5");
                            hp
                                    .DIV("ficheform-Choice")
                                    .INPUT_text(input)
                                    .__space()
                                    .LABEL_for(input.id())
                                    .__escape(CorpusMetadataUtils.getFicheTitle(entry.getFicheMeta(), workingLang, formatLocale))
                                    ._LABEL()
                                    ._DIV();
                        }
                    }
                });
        return true;
    }


    private static boolean printFicheStyleMotcleList(HtmlPrinter hp, ThesaurusIncludeElement.FicheStyle formElement, String name, UserLangContext userLangContext) {
        List<ThesaurusIncludeElement.Entry> entryList = formElement.getEntryList();
        if (entryList.isEmpty()) {
            return false;
        }
        int idalphaStyle = InputItem.getStyleValue(formElement.getIdalphaStyle());
        hp
                .__(FICHE_LIST_CELL, () -> {
                    if (hp.isWithJavascript()) {
                        for (ThesaurusIncludeElement.Entry entry : entryList) {
                            printFicheListItem(hp, name, entry.getValue(), entry.getPoids(), geFicheStyleLabel(entry.getMotcle(), userLangContext, idalphaStyle));
                        }
                    } else {
                        for (ThesaurusIncludeElement.Entry entry : entryList) {
                            HtmlAttributes input = hp.name(name).value(entry.getValue()).size("5");
                            hp
                                    .DIV("ficheform-Choice")
                                    .INPUT_text(input)
                                    .__space()
                                    .LABEL_for(input.id())
                                    .__escape(geFicheStyleLabel(entry.getMotcle(), userLangContext, idalphaStyle))
                                    ._LABEL()
                                    ._DIV();
                        }
                    }
                });
        return true;
    }

    private static boolean printFicheListItem(HtmlPrinter hp, String name, String value, int poids, String label) {
        HtmlAttributes checkbox = hp.name(name).value(value).checked(true).attr("data-ficheform-role", "fiche-checkbox");
        hp
                .DIV(HA.attr("data-ficheform-role", "fiche-listitem").classes("ficheform-Choice"))
                .INPUT_checkbox(checkbox)
                .LABEL_for(checkbox.id());
        if (poids > 0) {
            hp
                    .__space()
                    .SPAN(HA.attr("data-ficheform-role", "poids"));
            if (poids > 1) {
                hp
                        .__escape('<')
                        .__append(poids)
                        .__escape('>');
            }
            hp
                    ._SPAN();
        }
        hp
                .__space()
                .__escape(label)
                ._LABEL()
                ._DIV();
        return true;
    }


    private static String geFicheStyleLabel(Motcle motcle, UserLangContext userLangContext, int idalphaStyle) {
        return FichothequeUtils.getNumberPhrase(motcle, MetadataPhraseDefCatalog.FICHESTYLE_NAME, userLangContext.getWorkingLang(), userLangContext.getFormatLocale()) + " – " + InputItem.getText(motcle, userLangContext.getWorkingLang(), idalphaStyle);
    }


}
