/* FichothequeLib_API - Copyright (c) 2015-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.text.Collator;
import java.util.Comparator;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.thesaurus.Motcle;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
public final class Comparators {

    public final static Comparator<Motcle> IDALPHA = new IdalphaComparator();
    public final static Comparator<FicheMeta> FICHEID = new FicheIdComparator();
    public final static Comparator<FicheMeta> CREATIONDATE = new CreationDateComparator();
    public final static Comparator<FicheMeta> MODIFICATIONDATE = new ModificationDateComparator();
    public final static Comparator<Liaison> LIAISONID = new LiaisonIdComparator();
    public final static Comparator<FieldKey> FIELDKEY = new FieldKeyComparator();
    public final static Comparator<Subset> SIZE = new SizeComparator();

    private Comparators() {

    }

    public static Comparator<FicheMeta> titre(Collator collator) {
        return new TitreComparator(collator);
    }

    public static Comparator<Motcle> label(Lang lang) {
        Collator collator = Collator.getInstance(lang.toLocale());
        collator.setStrength(Collator.PRIMARY);
        return new LabelComparator(lang, collator);
    }

    public static Comparator<Motcle> label(Lang lang, Collator collator) {
        return new LabelComparator(lang, collator);
    }

    private static int compareId(SubsetItem piece1, SubsetItem piece2) {
        int result = piece1.getSubset().getSubsetKey().compareTo(piece2.getSubset().getSubsetKey());
        if (result != 0) {
            return result;
        }
        int id1 = piece1.getId();
        int id2 = piece2.getId();
        if (id1 < id2) {
            return -1;
        }
        if (id1 > id2) {
            return 1;
        }
        return 0;
    }

    private static String toComparableString(Motcle motcle, Lang lang) {
        Label label = motcle.getLabels().getLabel(lang);
        if (label != null) {
            return label.getLabelString();
        } else {
            return "";
        }
    }


    private static class FicheIdComparator implements Comparator<FicheMeta> {

        private FicheIdComparator() {

        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            return compareId(ficheMeta1, ficheMeta2);
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof FicheIdComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1001;
        }

    }


    private static class IdalphaComparator implements Comparator<Motcle> {

        private IdalphaComparator() {
        }

        @Override
        public int compare(Motcle motcle1, Motcle motcle2) {
            String idalpha1 = motcle1.getIdalpha();
            String idalpha2 = motcle2.getIdalpha();
            return idalpha1.compareTo(idalpha2);
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof IdalphaComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1002;
        }

    }


    private static class CreationDateComparator implements Comparator<FicheMeta> {

        private CreationDateComparator() {
        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            int resultat = -FuzzyDate.compare(ficheMeta1.getCreationDate(), ficheMeta2.getCreationDate());
            if (resultat == 0) {
                resultat = -compareId(ficheMeta1, ficheMeta2);
            }
            return resultat;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof CreationDateComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1003;
        }

    }


    private static class ModificationDateComparator implements Comparator<FicheMeta> {

        private ModificationDateComparator() {
        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            int resultat = -FuzzyDate.compare(ficheMeta1.getModificationDate(), ficheMeta2.getModificationDate());
            if (resultat == 0) {
                resultat = -FuzzyDate.compare(ficheMeta1.getCreationDate(), ficheMeta2.getCreationDate());
                if (resultat == 0) {
                    resultat = -compareId(ficheMeta1, ficheMeta2);
                }
            }
            return resultat;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof ModificationDateComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1004;
        }

    }


    private static class TitreComparator implements Comparator<FicheMeta> {

        private final Collator collator;

        private TitreComparator(Collator collator) {
            this.collator = collator;
        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            int resultat = collator.compare(ficheMeta1.getTitre(), ficheMeta2.getTitre());
            if (resultat == 0) {
                resultat = compareId(ficheMeta1, ficheMeta2);
            }
            return resultat;
        }

    }


    private static class LabelComparator implements Comparator<Motcle> {

        private final Lang lang;
        private final Collator collator;

        public LabelComparator(Lang lang, Collator collator) {
            this.lang = lang;
            this.collator = collator;
        }

        @Override
        public int compare(Motcle motcle1, Motcle motcle2) {
            int resultat = collator.compare(toComparableString(motcle1, lang), toComparableString(motcle2, lang));
            if (resultat == 0) {
                resultat = compareId(motcle1, motcle2);
            }
            return resultat;
        }

    }


    private static class FieldKeyComparator implements Comparator<FieldKey> {

        private FieldKeyComparator() {
        }

        @Override
        public int compare(FieldKey fieldKey1, FieldKey fieldKey2) {
            if (fieldKey1.equals(fieldKey2)) {
                return 0;
            }
            if (fieldKey1.equals(FieldKey.TITRE)) {
                return -1;
            }
            if (fieldKey2.equals(FieldKey.TITRE)) {
                return 1;
            }
            if (fieldKey1.equals(FieldKey.SOUSTITRE)) {
                return -1;
            }
            if (fieldKey2.equals(FieldKey.SOUSTITRE)) {
                return 1;
            }
            if (fieldKey1.equals(FieldKey.REDACTEURS)) {
                return -1;
            }
            if (fieldKey2.equals(FieldKey.REDACTEURS)) {
                return 1;
            }
            if (fieldKey1.equals(FieldKey.LANG)) {
                return -1;
            }
            if (fieldKey2.equals(FieldKey.LANG)) {
                return 1;
            }
            if (fieldKey1.isProprieteFieldKey()) {
                if (fieldKey2.isProprieteFieldKey()) {
                    return fieldKey1.toString().compareTo(fieldKey2.toString());
                } else {
                    return -1;
                }
            }
            if (fieldKey2.isProprieteFieldKey()) {
                return 1;
            }
            if (fieldKey1.isInformationFieldKey()) {
                if (fieldKey2.isInformationFieldKey()) {
                    return fieldKey1.toString().compareTo(fieldKey2.toString());
                } else {
                    return -1;
                }
            }
            if (fieldKey2.isInformationFieldKey()) {
                return 1;
            }
            if (fieldKey1.isSectionFieldKey()) {
                if (fieldKey2.isSectionFieldKey()) {
                    return fieldKey1.toString().compareTo(fieldKey2.toString());
                } else {
                    return -1;
                }
            }
            if (fieldKey2.isSectionFieldKey()) {
                return 1;
            }
            return fieldKey1.toString().compareTo(fieldKey2.toString());
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof FieldKeyComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1005;
        }

    }


    private static class LiaisonIdComparator implements Comparator<Liaison> {

        private LiaisonIdComparator() {

        }

        @Override
        public int compare(Liaison liaison1, Liaison liaison2) {
            return compareId(liaison1.getSubsetItem(), liaison2.getSubsetItem());
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof FicheIdComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1006;
        }

    }


    private static class SizeComparator implements Comparator<Subset> {

        private SizeComparator() {

        }

        @Override
        public int compare(Subset subset1, Subset subset2) {
            int size1 = subset1.size();
            int size2 = subset2.size();
            if (size1 > size2) {
                return -1;
            }
            if (size1 < size2) {
                return 1;
            }
            return subset1.getSubsetKey().compareTo(subset2.getSubsetKey());
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof SizeComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1007;
        }

    }

}
