/* FichothequeLib_API - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.RandomAccess;
import java.util.Set;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.album.Album;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.SortConstants;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.selection.RedacteurQuery;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.primitives.Range;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionUtils {

    public final static List<FicheQuery> EMPTY_FICHEQUERYLIST = Collections.emptyList();
    public final static List<MotcleQuery> EMPTY_MOTCLEQUERYLIST = Collections.emptyList();
    public final static List<IllustrationQuery> EMPTY_ILLUSTRATIONQUERYLIST = Collections.emptyList();
    public final static List<DocumentQuery> EMPTY_DOCUMENTQUERYLIST = Collections.emptyList();
    public final static DocumentQuery EMPTY_DOCUMENTQUERY = new EmptyDocumentQuery();
    public final static MotcleQuery EMPTY_MOTCLEQUERY = new EmptyMotcleQuery();
    public final static IllustrationQuery EMPTY_ILLUSTRATIONQUERY = new EmptyIllustrationQuery();
    public final static FicheQuery EMPTY_FICHEQUERY = new EmptyFicheQuery();
    public final static RedacteurQuery EMPTY_REDACTEURQUERY = new EmptyRedacteurQuery();
    public final static MotcleSelector NONE_MOTCLESELECTOR = new NoneMotcleSelector();

    private SelectionUtils() {
    }

    public static String fieldConditionToString(short fieldConditionScope) {
        switch (fieldConditionScope) {
            case FieldQuery.TITRE_SCOPE:
                return "titre";
            case FieldQuery.ENTETE_SCOPE:
                return "entete";
            case FieldQuery.FICHE_SCOPE:
                return "fiche";
            case FieldQuery.SELECTION_SCOPE:
                return "selection";
            default:
                throw new IllegalArgumentException("Unknwon scope: " + fieldConditionScope);
        }
    }

    public static short fieldConditionToShort(String fieldConditionScope) {
        switch (fieldConditionScope) {
            case "titre":
                return FieldQuery.TITRE_SCOPE;
            case "entete":
                return FieldQuery.ENTETE_SCOPE;
            case "fiche":
                return FieldQuery.FICHE_SCOPE;
            case "selection":
                return FieldQuery.SELECTION_SCOPE;
            default:
                throw new IllegalArgumentException("Unknwon scope: " + fieldConditionScope);
        }
    }

    public static String motcleFieldConditionScopeToString(short motcleFieldConditionScope) {
        switch (motcleFieldConditionScope) {
            case MotcleQuery.ONLY_IDALPHA_SCOPE:
                return "idalpha_only";
            case MotcleQuery.WITH_IDALPHA_SCOPE:
                return "idalpha_with";
            case MotcleQuery.WITHOUT_IDALPHA_SCOPE:
                return "idalpha_without";
            default:
                throw new IllegalArgumentException("Unknwon scope: " + motcleFieldConditionScope);
        }
    }

    public static short motcleFieldConditionScopeToShort(String motcleFieldConditionScope) {
        switch (motcleFieldConditionScope) {
            case "idalpha_only":
                return MotcleQuery.ONLY_IDALPHA_SCOPE;
            case "idalpha_with":
                return MotcleQuery.WITH_IDALPHA_SCOPE;
            case "idalpha_without":
                return MotcleQuery.WITHOUT_IDALPHA_SCOPE;
            default:
                throw new IllegalArgumentException("Unknwon scope: " + motcleFieldConditionScope);
        }
    }

    public static String checkSortType(String sortType) {
        switch (sortType) {
            case SortConstants.NONE:
            case SortConstants.TITRE:
            case SortConstants.ID:
            case SortConstants.CREATIONDATE:
            case SortConstants.MODIFICATIONDATE:
                return sortType;
            case "idcorpus":
                return SortConstants.ID;
            default:
                throw new IllegalArgumentException("Unknown sortType: " + sortType);
        }
    }

    public static String checkDiscardFilter(String discardFilter) {
        switch (discardFilter) {
            case FicheQuery.ALL_DISCARDFILTER:
            case FicheQuery.NONE_DISCARDFILTER:
            case FicheQuery.ONLY_DISCARDFILTER:
                return discardFilter;
            default:
                throw new IllegalArgumentException("Unknown discardFilter: " + discardFilter);
        }
    }

    public static String checkRedacteurFilter(String redacteurFilter) {
        switch (redacteurFilter) {
            case FicheQuery.ALL_REDACTEURFILTER:
            case FicheQuery.NONE_REDACTEURFILTER:
            case FicheQuery.ANY_REDACTEURFILTER:
            case FicheQuery.SOME_REDACTEURFILTER:
                return redacteurFilter;
            default:
                throw new IllegalArgumentException("Unknown redacteurFilter: " + redacteurFilter);
        }
    }

    public static Corpus[] toCorpusArray(Fichotheque fichotheque, FicheQuery ficheQuery, Predicate<Subset> subsetAccessPredicate) {
        if (ficheQuery == null) {
            return FichothequeUtils.toCorpusArray(fichotheque, subsetAccessPredicate);
        }
        List<SubsetKey> corpusKeyList = ficheQuery.getCorpusKeyList();
        if (corpusKeyList.isEmpty()) {
            return FichothequeUtils.toCorpusArray(fichotheque, subsetAccessPredicate);
        }
        List<Corpus> list = new ArrayList<Corpus>();
        for (SubsetKey corpusKey : corpusKeyList) {
            if (corpusKey == null) {
                continue;
            }
            if (!corpusKey.isCorpusSubset()) {
                continue;
            }
            Corpus corpus = (Corpus) fichotheque.getSubset(corpusKey);
            if ((corpus != null) && (subsetAccessPredicate.test(corpus))) {
                list.add(corpus);
            }
        }
        return list.toArray(new Corpus[list.size()]);
    }

    public static Thesaurus[] toThesaurusArray(Fichotheque fichotheque, MotcleQuery motcleQuery, Predicate<Subset> subsetAccessPredicate) {
        List<SubsetKey> subsetKeyList = motcleQuery.getThesaurusKeyList();
        if (subsetKeyList.isEmpty()) {
            return FichothequeUtils.toThesaurusArray(fichotheque, subsetAccessPredicate);
        }
        List<Thesaurus> list = new ArrayList<Thesaurus>();
        for (SubsetKey subsetKey : subsetKeyList) {
            Thesaurus thesaurus = (Thesaurus) fichotheque.getSubset(subsetKey);
            if ((thesaurus != null) && (subsetAccessPredicate.test(thesaurus))) {
                list.add(thesaurus);
            }
        }
        return list.toArray(new Thesaurus[list.size()]);
    }

    public static Album[] toAlbumArray(Fichotheque fichotheque, IllustrationQuery illustrationQuery, Predicate<Subset> subsetAccessPredicate) {
        List<SubsetKey> subsetKeyList = illustrationQuery.getAlbumKeyList();
        if (subsetKeyList.isEmpty()) {
            return FichothequeUtils.toAlbumArray(fichotheque, subsetAccessPredicate);
        }
        List<Album> list = new ArrayList<Album>();
        for (SubsetKey subsetKey : subsetKeyList) {
            Album album = (Album) fichotheque.getSubset(subsetKey);
            if ((album != null) && (subsetAccessPredicate.test(album))) {
                list.add(album);
            }
        }
        return list.toArray(new Album[list.size()]);
    }

    public static Addenda[] toAddendaArray(Fichotheque fichotheque, DocumentQuery documentQuery, Predicate<Subset> subsetAccessPredicate) {
        List<SubsetKey> subsetKeyList = documentQuery.getAddendaKeyList();
        if (subsetKeyList.isEmpty()) {
            return FichothequeUtils.toAddendaArray(fichotheque, subsetAccessPredicate);
        }
        List<Addenda> list = new ArrayList<Addenda>();
        for (SubsetKey subsetKey : subsetKeyList) {
            Addenda addenda = (Addenda) fichotheque.getSubset(subsetKey);
            if ((addenda != null) && (subsetAccessPredicate.test(addenda))) {
                list.add(addenda);
            }
        }
        return list.toArray(new Addenda[list.size()]);
    }

    public static Sphere[] toSphereArray(Fichotheque fichotheque, RedacteurQuery redacteurQuery, Predicate<Subset> subsetAccessPredicate) {
        List<SubsetKey> subsetKeyList = redacteurQuery.getSphereKeyList();
        if (subsetKeyList.isEmpty()) {
            return FichothequeUtils.toSphereArray(fichotheque, subsetAccessPredicate);
        }
        List<Sphere> list = new ArrayList<Sphere>();
        for (SubsetKey subsetKey : subsetKeyList) {
            Sphere sphere = (Sphere) fichotheque.getSubset(subsetKey);
            if ((sphere != null) && (subsetAccessPredicate.test(sphere))) {
                list.add(sphere);
            }
        }
        return list.toArray(new Sphere[list.size()]);
    }


    public static List<FicheQuery> wrap(FicheQuery[] array) {
        return new FicheQueryList(array);
    }

    public static boolean isEmpty(FicheQuery ficheQuery) {
        if (!ficheQuery.getCorpusKeyList().isEmpty()) {
            return false;
        }
        return isOnlyList(ficheQuery);
    }

    public static boolean isEmpty(MotcleQuery motcleQuery) {
        if (!motcleQuery.getThesaurusKeyList().isEmpty()) {
            return false;
        }
        return isOnlyList(motcleQuery);
    }

    public static boolean isEmpty(IllustrationQuery illustrationQuery) {
        if (!illustrationQuery.getAlbumKeyList().isEmpty()) {
            return false;
        }
        if (!illustrationQuery.getAlbumDimNameList().isEmpty()) {
            return false;
        }
        if (illustrationQuery.getCroisementCondition() != null) {
            return false;
        }
        return true;
    }

    public static boolean isEmpty(DocumentQuery documentQuery) {
        if (!documentQuery.getAddendaKeyList().isEmpty()) {
            return false;
        }
        if (documentQuery.getCroisementCondition() != null) {
            return false;
        }
        return true;
    }

    public static boolean isOnlyList(FicheQuery ficheQuery) {
        if (ficheQuery.getIdRanges() != null) {
            return false;
        }
        if (ficheQuery.getMotcleCondition() != null) {
            return false;
        }
        if (ficheQuery.getFieldQuery() != null) {
            return false;
        }
        if (!ficheQuery.getRedacteurFilter().equals(FicheQuery.ALL_REDACTEURFILTER)) {
            return false;
        }
        if (ficheQuery.getCroisementCondition() != null) {
            return false;
        }
        if (ficheQuery.isWithGeoloc()) {
            return false;
        }
        if (ficheQuery.getPeriodCondition() != null) {
            return false;
        }
        return true;
    }

    public static boolean isOnlyList(MotcleQuery motcleQuery) {
        if (motcleQuery.getIdRanges() != null) {
            return false;
        }
        if (motcleQuery.getLevelRanges() != null) {
            return false;
        }
        if (motcleQuery.getContentCondition() != null) {
            return false;
        }
        if (motcleQuery.getFicheCondition() != null) {
            return false;
        }
        if (motcleQuery.getCroisementCondition() != null) {
            return false;
        }
        return true;
    }

    public static CroisementCondition toCroisementCondition(String mode, int poids) {
        String[] lienModeArray = {mode};
        Ranges ranges = null;
        if (poids > 0) {
            ranges = new Ranges();
            ranges.addRange(new Range(poids, poids));
        }
        return new InternalCroisementCondition(lienModeArray, ranges, false);
    }

    public static CroisementCondition toCroisementCondition(String mode, Ranges ranges, boolean isExclude) {
        String[] lienModeArray = {mode};
        return new InternalCroisementCondition(lienModeArray, ranges, isExclude);
    }

    public static Predicate<Motcle> toPredicate(MotcleSelector motcleSelector) {
        Set<Motcle> motcleSet = new HashSet<Motcle>();
        for (Thesaurus thesaurus : motcleSelector.getThesaurusList()) {
            for (Motcle motcle : thesaurus.getMotcleList()) {
                if (motcleSelector.test(motcle)) {
                    motcleSet.add(motcle);
                }
            }
        }
        return EligibilityUtils.toMotclePredicate(motcleSet);
    }

    public static Predicate<FicheMeta> toPredicate(FicheSelector ficheSelector) {
        Set<FicheMeta> ficheMetaSet = new HashSet<FicheMeta>();
        for (Corpus corpus : ficheSelector.getCorpusList()) {
            for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                if (ficheSelector.test(ficheMeta)) {
                    ficheMetaSet.add(ficheMeta);
                }
            }
        }
        return EligibilityUtils.toFichePredicate(ficheMetaSet);
    }

    public static FicheQuery derive(FicheQuery ficheQuery, Collection<SubsetKey> corpusKeys) {
        List<SubsetKey> templist = new ArrayList<SubsetKey>();
        for (SubsetKey subsetKey : corpusKeys) {
            if (subsetKey != null) {
                if (subsetKey.isCorpusSubset()) {
                    templist.add(subsetKey);
                }
            }
        }
        List<SubsetKey> list = FichothequeUtils.toList(templist);
        return new DerivationFicheQuery(ficheQuery, list);
    }

    public static MotcleQuery derive(MotcleQuery motcleQuery, Collection<SubsetKey> thesaurusKeys) {
        List<SubsetKey> templist = new ArrayList<SubsetKey>();
        for (SubsetKey subsetKey : thesaurusKeys) {
            if (subsetKey != null) {
                if (subsetKey.isThesaurusSubset()) {
                    templist.add(subsetKey);
                }
            }
        }
        List<SubsetKey> list = FichothequeUtils.toList(templist);
        return new DerivationMotcleQuery(motcleQuery, list);
    }


    private static class FicheQueryList extends AbstractList<FicheQuery> implements RandomAccess {

        private final FicheQuery[] array;

        private FicheQueryList(FicheQuery[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public FicheQuery get(int index) {
            return array[index];
        }

    }

    public static List<MotcleQuery> wrap(MotcleQuery[] array) {
        return new MotcleQueryList(array);
    }

    public static MotcleQuery deriveWithoutContentCondition(MotcleQuery motcleQuery) {
        return new WithouContentConditionMotcleQuery(motcleQuery);
    }


    private static class MotcleQueryList extends AbstractList<MotcleQuery> implements RandomAccess {

        private final MotcleQuery[] array;

        private MotcleQueryList(MotcleQuery[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public MotcleQuery get(int index) {
            return array[index];
        }

    }


    public static List<IllustrationQuery> wrap(IllustrationQuery[] array) {
        return new IllustrationQueryList(array);
    }


    private static class IllustrationQueryList extends AbstractList<IllustrationQuery> implements RandomAccess {

        private final IllustrationQuery[] array;

        private IllustrationQueryList(IllustrationQuery[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public IllustrationQuery get(int index) {
            return array[index];
        }

    }

    public static List<DocumentQuery> wrap(DocumentQuery[] array) {
        return new DocumentQueryList(array);
    }

    public static FicheCondition toFicheCondition(List<FicheQuery> ficheQueryList, short selectMode) {
        if (ficheQueryList.isEmpty()) {
            throw new IllegalArgumentException("ficheQueryList is empty");
        }
        return new InternalFicheCondition(SelectionUtils.wrap(ficheQueryList.toArray(new FicheQuery[ficheQueryList.size()])), selectMode);
    }

    public static MotcleCondition toMotcleCondition(List<MotcleQuery> motcleQueryList, short selectMode) {
        if (motcleQueryList.isEmpty()) {
            throw new IllegalArgumentException("motcleQueryList is empty");
        }
        return new InternalMotcleCondition(SelectionUtils.wrap(motcleQueryList.toArray(new MotcleQuery[motcleQueryList.size()])), selectMode);
    }


    private static class DocumentQueryList extends AbstractList<DocumentQuery> implements RandomAccess {

        private final DocumentQuery[] array;

        private DocumentQueryList(DocumentQuery[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public DocumentQuery get(int index) {
            return array[index];
        }

    }


    private static class EmptyDocumentQuery implements DocumentQuery {

        private EmptyDocumentQuery() {
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return null;
        }

        @Override
        public List<SubsetKey> getAddendaKeyList() {
            return FichothequeUtils.EMPTY_SUBSETKEYLIST;
        }

        @Override
        public Condition getNameCondition() {
            return null;
        }

    }


    private static class EmptyMotcleQuery implements MotcleQuery {

        private EmptyMotcleQuery() {
        }

        @Override
        public Ranges getIdRanges() {
            return null;
        }

        @Override
        public Ranges getLevelRanges() {
            return null;
        }

        @Override
        public List<SubsetKey> getThesaurusKeyList() {
            return FichothequeUtils.EMPTY_SUBSETKEYLIST;
        }

        @Override
        public Condition getContentCondition() {
            return null;
        }

        @Override
        public short getContentConditionScope() {
            return MotcleQuery.WITH_IDALPHA_SCOPE;
        }

        @Override
        public FicheCondition getFicheCondition() {
            return null;
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return null;
        }

        @Override
        public boolean includeSatellites() {
            return false;
        }

    }


    private static class EmptyIllustrationQuery implements IllustrationQuery {

        private EmptyIllustrationQuery() {
        }

        @Override
        public List<SubsetKey> getAlbumKeyList() {
            return FichothequeUtils.EMPTY_SUBSETKEYLIST;
        }

        @Override
        public List<String> getAlbumDimNameList() {
            return StringUtils.EMPTY_STRINGLIST;
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return null;
        }

    }


    private static class EmptyFicheQuery implements FicheQuery {

        private EmptyFicheQuery() {
        }

        @Override
        public List<SubsetKey> getCorpusKeyList() {
            return FichothequeUtils.EMPTY_SUBSETKEYLIST;
        }

        @Override
        public Ranges getIdRanges() {
            return null;
        }

        @Override
        public MotcleCondition getMotcleCondition() {
            return null;
        }

        @Override
        public FieldQuery getFieldQuery() {
            return null;
        }

        @Override
        public String getRedacteurFilter() {
            return FicheQuery.ALL_REDACTEURFILTER;
        }

        @Override
        public List<String> getRedacteurGlobalIdList() {
            return StringUtils.EMPTY_STRINGLIST;
        }

        @Override
        public List<SubsetKey> getSphereKeyList() {
            return FichothequeUtils.EMPTY_SUBSETKEYLIST;
        }


        @Override
        public CroisementCondition getCroisementCondition() {
            return null;
        }

        @Override
        public boolean isWithRattachement() {
            return false;
        }

        @Override
        public PeriodCondition getPeriodCondition() {
            return null;
        }

        @Override
        public boolean isWithCurrentCorpus() {
            return false;
        }

        @Override
        public String getDiscardFilter() {
            return FicheQuery.ALL_DISCARDFILTER;
        }

        @Override
        public boolean isWithGeoloc() {
            return false;
        }

    }


    private static class EmptyRedacteurQuery implements RedacteurQuery {

        private EmptyRedacteurQuery() {
        }

        @Override
        public List<SubsetKey> getSphereKeyList() {
            return FichothequeUtils.EMPTY_SUBSETKEYLIST;
        }

        @Override
        public Condition getNomcompletCondition() {
            return null;
        }

    }


    private static class NoneMotcleSelector implements MotcleSelector {

        private NoneMotcleSelector() {

        }

        @Override
        public boolean test(Motcle motcle) {
            return false;
        }

        @Override
        public List<Thesaurus> getThesaurusList() {
            return FichothequeUtils.EMPTY_THESAURUSLIST;
        }

        @Override
        public Croisement isSelected(Motcle motcle, Croisement croisement) {
            return null;
        }

    }


    private static class WithouContentConditionMotcleQuery implements MotcleQuery {

        private final MotcleQuery motcleQuery;

        private WithouContentConditionMotcleQuery(MotcleQuery motcleQuery) {
            this.motcleQuery = motcleQuery;
        }

        @Override
        public Ranges getIdRanges() {
            return motcleQuery.getIdRanges();
        }

        @Override
        public Ranges getLevelRanges() {
            return motcleQuery.getLevelRanges();
        }

        @Override
        public List<SubsetKey> getThesaurusKeyList() {
            return motcleQuery.getThesaurusKeyList();
        }

        @Override
        public Condition getContentCondition() {
            return null;
        }

        @Override
        public short getContentConditionScope() {
            return motcleQuery.getContentConditionScope();
        }

        @Override
        public FicheCondition getFicheCondition() {
            return motcleQuery.getFicheCondition();
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return motcleQuery.getCroisementCondition();
        }

        @Override
        public boolean includeSatellites() {
            return motcleQuery.includeSatellites();
        }

    }


    private static class InternalFicheCondition implements FicheCondition {

        private final List<FicheQuery> ficheQueryList;
        private final short selectMode;

        private InternalFicheCondition(List<FicheQuery> ficheQueryList, short selectMode) {
            this.ficheQueryList = ficheQueryList;
            this.selectMode = selectMode;
        }

        @Override
        public List<FicheQuery> getFicheQueryList() {
            return ficheQueryList;
        }

        @Override
        public short getSelectMode() {
            return selectMode;
        }

    }


    private static class InternalMotcleCondition implements MotcleCondition {

        private final List<MotcleQuery> motcleQueryList;
        private final short selectMode;

        private InternalMotcleCondition(List<MotcleQuery> motcleQueryList, short selectMode) {
            this.motcleQueryList = motcleQueryList;
            this.selectMode = selectMode;
        }

        @Override
        public List<MotcleQuery> getMotcleQueryList() {
            return motcleQueryList;
        }

        @Override
        public short getSelectMode() {
            return selectMode;
        }

    }


    private static class InternalCroisementCondition implements CroisementCondition {

        private final String[] lienModeArray;
        private final boolean exclusion;
        private final Ranges poidsRanges;

        private InternalCroisementCondition(String[] lienModeArray, Ranges poidsRanges, boolean exclusion) {
            this.lienModeArray = lienModeArray;
            this.poidsRanges = poidsRanges;
            this.exclusion = exclusion;
        }

        @Override
        public String[] getLienModeArray() {
            return lienModeArray;
        }

        @Override
        public Ranges getPoidsRanges() {
            return poidsRanges;
        }

        @Override
        public boolean isExclusionRangeList() {
            return exclusion;
        }

    }


    private static class DerivationFicheQuery implements FicheQuery {

        private final FicheQuery ficheQuery;
        private final List<SubsetKey> corpusKeyList;

        private DerivationFicheQuery(FicheQuery ficheQuery, List<SubsetKey> corpusKeyList) {
            this.ficheQuery = ficheQuery;
            this.corpusKeyList = corpusKeyList;
        }


        @Override
        public List<SubsetKey> getCorpusKeyList() {
            return corpusKeyList;
        }

        @Override
        public Ranges getIdRanges() {
            return ficheQuery.getIdRanges();
        }

        @Override
        public MotcleCondition getMotcleCondition() {
            return ficheQuery.getMotcleCondition();
        }

        @Override
        public FieldQuery getFieldQuery() {
            return ficheQuery.getFieldQuery();
        }

        @Override
        public String getRedacteurFilter() {
            return ficheQuery.getRedacteurFilter();
        }

        @Override
        public List<String> getRedacteurGlobalIdList() {
            return ficheQuery.getRedacteurGlobalIdList();
        }

        @Override
        public List<SubsetKey> getSphereKeyList() {
            return ficheQuery.getSphereKeyList();
        }


        @Override
        public CroisementCondition getCroisementCondition() {
            return ficheQuery.getCroisementCondition();
        }

        @Override
        public boolean isWithRattachement() {
            return ficheQuery.isWithRattachement();
        }

        @Override
        public PeriodCondition getPeriodCondition() {
            return ficheQuery.getPeriodCondition();
        }

        @Override
        public boolean isWithCurrentCorpus() {
            return ficheQuery.isWithCurrentCorpus();
        }

        @Override
        public String getDiscardFilter() {
            return ficheQuery.getDiscardFilter();
        }

        @Override
        public boolean isWithGeoloc() {
            return ficheQuery.isWithGeoloc();
        }


    }


    private static class DerivationMotcleQuery implements MotcleQuery {

        private final MotcleQuery motcleQuery;
        private final List<SubsetKey> thesaurusKeyList;

        private DerivationMotcleQuery(MotcleQuery motcleQuery, List<SubsetKey> thesaurusKeyList) {
            this.motcleQuery = motcleQuery;
            this.thesaurusKeyList = thesaurusKeyList;
        }

        @Override
        public Ranges getIdRanges() {
            return motcleQuery.getIdRanges();
        }

        @Override
        public Ranges getLevelRanges() {
            return motcleQuery.getLevelRanges();
        }

        @Override
        public List<SubsetKey> getThesaurusKeyList() {
            return thesaurusKeyList;
        }

        @Override
        public Condition getContentCondition() {
            return motcleQuery.getContentCondition();
        }

        @Override
        public short getContentConditionScope() {
            return motcleQuery.getContentConditionScope();
        }

        @Override
        public FicheCondition getFicheCondition() {
            return motcleQuery.getFicheCondition();
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return motcleQuery.getCroisementCondition();
        }

        @Override
        public boolean includeSatellites() {
            return motcleQuery.includeSatellites();
        }

    }

}
