/* FichothequeLib_Impl - Copyright (c) 2006-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.Metadata;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.CorpusEditor;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.CorpusMetadataEditor;
import net.fichotheque.corpus.metadata.ExistingFieldKeyException;
import net.fichotheque.corpus.metadata.FieldGeneration;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.FieldOptionException;
import net.fichotheque.corpus.metadata.FieldOptions;
import net.fichotheque.corpus.metadata.FieldOptionsConstants;
import net.fichotheque.corpus.metadata.MandatoryFieldException;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.FieldOptionsUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.localisation.LangsUtils;
import net.mapeadores.util.money.Currencies;
import net.mapeadores.util.money.CurrenciesUtils;
import net.mapeadores.util.money.ExtendedCurrency;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.Labels;
import net.mapeadores.util.text.LabelsCache;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
class CorpusMetadataImpl extends AbstractMetadata implements CorpusMetadata {

    private final CorpusImpl corpus;
    private final Map<FieldKey, CorpusFieldImpl> corpusFieldMap = new HashMap<FieldKey, CorpusFieldImpl>();
    private final List<CorpusFieldImpl> proprieteFieldList = new ArrayList<CorpusFieldImpl>();
    private final List<CorpusFieldImpl> informationFieldList = new ArrayList<CorpusFieldImpl>();
    private final List<CorpusFieldImpl> sectionFieldList = new ArrayList<CorpusFieldImpl>();
    private CorpusField geolocalisationField;
    private List<CorpusField> proprieteCacheList;
    private List<CorpusField> informationCacheList;
    private List<CorpusField> sectionCacheList;
    private FieldGeneration fieldGeneration = CorpusMetadataUtils.EMPTY_FIELDGENERATION;

    private CorpusMetadataImpl(CorpusImpl corpus) {
        super(corpus);
        this.corpus = corpus;
        try {
            innerCreateCorpusField(FieldKey.ID, CorpusField.NO_FICHEITEM_FIELD);
            innerCreateCorpusField(FieldKey.LANG, CorpusField.NO_FICHEITEM_FIELD);
            innerCreateCorpusField(FieldKey.TITRE, CorpusField.NO_FICHEITEM_FIELD);
            innerCreateCorpusField(FieldKey.REDACTEURS, CorpusField.NO_FICHEITEM_FIELD);
        } catch (ExistingFieldKeyException efke) {
        }
    }

    static CorpusMetadataImpl.InitEditor fromInit(CorpusImpl.InitEditor corpusInitEditor) {
        CorpusMetadataImpl metadata = new CorpusMetadataImpl((CorpusImpl) corpusInitEditor.getCorpus());
        return new InitEditor(metadata, corpusInitEditor);
    }

    static CorpusMetadataImpl fromNew(CorpusImpl corpus) {
        return new CorpusMetadataImpl(corpus);
    }

    @Override
    public Corpus getCorpus() {
        return corpus;
    }

    @Override
    public CorpusField getCorpusField(FieldKey fieldKey) {
        return corpusFieldMap.get(fieldKey);
    }

    @Override
    public List<CorpusField> getProprieteList() {
        List<CorpusField> result = proprieteCacheList;
        if (result == null) {
            result = initProprieteList();
        }
        return result;
    }

    @Override
    public List<CorpusField> getInformationList() {
        List<CorpusField> result = informationCacheList;
        if (result == null) {
            result = initInformationList();
        }
        return result;
    }

    @Override
    public List<CorpusField> getSectionList() {
        List<CorpusField> result = sectionCacheList;
        if (result == null) {
            result = initSectionList();
        }
        return result;
    }

    @Override
    public CorpusField getGeolocalisationField() {
        return geolocalisationField;
    }

    @Override
    public FieldGeneration getFieldGeneration() {
        return fieldGeneration;
    }

    CorpusMetadataEditor getCorpusMetadataEditor(CorpusImpl.CorpusEditorImpl corpusEditor) {
        return new CorpusMetadataEditorImpl(corpusEditor);
    }

    void clear() {
        sectionFieldList.clear();
        proprieteFieldList.clear();
        informationFieldList.clear();
        corpusFieldMap.clear();
        clearCache();
    }

    private Listeners getListeners() {
        return corpus.getListeners();
    }

    private void clearCache() {
        proprieteCacheList = null;
        informationCacheList = null;
        sectionCacheList = null;
    }

    @Override
    protected void fireChange(FichothequeEditor fichothequeEditor) {
        CorpusImpl.CorpusEditorImpl corpusEditor = (CorpusImpl.CorpusEditorImpl) fichothequeEditor.getCorpusEditor(corpus.getSubsetKey());
        corpusEditor.setMetadataChange();
    }

    private CorpusFieldImpl testCorpusField(CorpusField corpusField) throws IllegalArgumentException {
        if (!(corpusField instanceof CorpusFieldImpl)) {
            throw new IllegalArgumentException("corpusField argument does not come from this CorpusMetadata instance");
        }
        CorpusFieldImpl def = (CorpusFieldImpl) corpusField;
        if (def.getCorpusMetadata() != this) {
            throw new IllegalArgumentException("corpusField argument does not come from this CorpusMetadata instance");
        }
        return def;
    }

    private synchronized List<CorpusField> initProprieteList() {
        if (proprieteCacheList != null) {
            return proprieteCacheList;
        }
        List<CorpusField> list = CorpusMetadataUtils.wrap(proprieteFieldList.toArray(new CorpusField[proprieteFieldList.size()]));
        proprieteCacheList = list;
        return list;
    }

    private synchronized List<CorpusField> initInformationList() {
        if (informationCacheList != null) {
            return informationCacheList;
        }
        List<CorpusField> list = CorpusMetadataUtils.wrap(informationFieldList.toArray(new CorpusField[informationFieldList.size()]));
        informationCacheList = list;
        return list;
    }

    private synchronized List<CorpusField> initSectionList() {
        if (sectionCacheList != null) {
            return sectionCacheList;
        }
        List<CorpusField> list = CorpusMetadataUtils.wrap(sectionFieldList.toArray(new CorpusField[sectionFieldList.size()]));
        sectionCacheList = list;
        return list;
    }


    private class CorpusFieldImpl implements CorpusField {

        private final FieldKey fieldKey;
        private final short ficheItemType;
        private final LabelsCache labelsCache = new LabelsCache();
        private final Map<String, String> stringOptions = new HashMap<String, String>();
        private final InternalFieldOptions fieldOptions;
        private boolean generated;

        private CorpusFieldImpl(FieldKey fieldKey) {
            this(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
        }

        private CorpusFieldImpl(FieldKey fieldKey, short ficheItemType) {
            this.fieldKey = fieldKey;
            this.ficheItemType = ficheItemType;
            Short informationDisplay = null;
            if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.INFORMATIONDISPLAY_OPTION, fieldKey, ficheItemType)) {
                informationDisplay = FieldOptionsUtils.getDefaultInformationDisplay();
            }
            Short langScope = null;
            if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.LANGSCOPE_OPTION, fieldKey, ficheItemType)) {
                langScope = FieldOptionsUtils.getDefaultLangScope(fieldKey);
            }
            this.fieldOptions = new InternalFieldOptions(informationDisplay, langScope);
        }

        @Override
        public FieldKey getFieldKey() {
            return fieldKey;
        }

        @Override
        public FieldOptions getFieldOptions() {
            return fieldOptions;
        }

        @Override
        public CorpusMetadata getCorpusMetadata() {
            return CorpusMetadataImpl.this;
        }

        @Override
        public short getFicheItemType() {
            return ficheItemType;
        }

        @Override
        public String getStringOption(String optionName) {
            return stringOptions.get(optionName);
        }

        @Override
        public String[] getStringOptionNameArray() {
            int size = stringOptions.size();
            String[] result = new String[size];
            if (size == 0) {
                return result;
            }
            int p = 0;
            for (String key : stringOptions.keySet()) {
                result[p] = key;
                p++;
            }
            return result;
        }

        @Override
        public Labels getLabels() {
            return labelsCache.getLabels();
        }

        @Override
        public boolean isGenerated() {
            return generated;
        }

        private boolean putLabel(Label label) {
            return labelsCache.putLabel(label);
        }

        private boolean removeLabel(Lang lang) {
            return labelsCache.removeLabel(lang);
        }

        private boolean setOption(String optionName, Object optionValue) throws FieldOptionException {
            FieldOptionsUtils.checkFieldOption(optionName, fieldKey, ficheItemType);
            if (optionName.equals(FieldOptionsConstants.INFORMATIONDISPLAY_OPTION)) {
                return fieldOptions.setInformationDisplay(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION)) {
                return fieldOptions.setDefaultSphereKey(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.SUBFIELDDISPLAY_OPTION)) {
                return fieldOptions.setSubfieldDisplay(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.CURRENCYARRAY_OPTION)) {
                return fieldOptions.setCurrencies(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION)) {
                return fieldOptions.setAddressFieldList(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.LANGSCOPE_OPTION)) {
                return fieldOptions.setLangScope(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.LANGARRAY_OPTION)) {
                return fieldOptions.setLangs(optionValue);
            } else {
                if ((optionValue != null) && (!(optionValue instanceof String))) {
                    throw new FieldOptionException("optionValue must be null or a String instance");
                }
                return setStringOption(optionName, (String) optionValue);
            }
        }

        private boolean setStringOption(String optionName, String optionValue) {
            if (optionValue != null) {
                optionValue = optionValue.trim();
                if (optionValue.length() == 0) {
                    optionValue = null;
                }
            }
            if (optionValue == null) {
                if (stringOptions.containsKey(optionName)) {
                    stringOptions.remove(optionName);
                    return true;
                } else {
                    return false;
                }
            }
            String currentValue = stringOptions.get(optionName);
            if (currentValue == null) {
                stringOptions.put(optionName, optionValue);
                return true;
            } else if (currentValue.equals(optionValue)) {
                return false;
            } else {
                stringOptions.put(optionName, optionValue);
                return true;
            }
        }


        private class InternalFieldOptions implements FieldOptions {

            private Short informationDisplay;
            private Short langScope;
            private SubsetKey sphereKey = null;
            private boolean subfieldDisplay = false;
            private Currencies currencies = null;
            private List<CorpusField> addressFieldList = null;
            private Langs langs = null;

            private InternalFieldOptions(Short informationDisplay, Short langScope) {
                this.informationDisplay = informationDisplay;
                this.langScope = langScope;
            }

            @Override
            public SubsetKey getDefaultSphereKey() {
                return sphereKey;
            }

            @Override
            public Currencies getCurrencies() {
                return currencies;
            }

            @Override
            public List<CorpusField> getAddressFieldList() {
                return addressFieldList;
            }

            @Override
            public Short getInformationDisplay() {
                return informationDisplay;
            }

            @Override
            public boolean getSubfieldDisplay() {
                return subfieldDisplay;
            }

            @Override
            public Short getLangScope() {
                return langScope;
            }

            @Override
            public Langs getLangs() {
                return langs;
            }

            private boolean setSubfieldDisplay(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (subfieldDisplay) {
                        subfieldDisplay = false;
                        return true;
                    } else {
                        return false;
                    }
                } else if (optionValue.equals(Boolean.TRUE)) {
                    if (!subfieldDisplay) {
                        subfieldDisplay = true;
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    throw new FieldOptionException("optionValue must be null or equals to Boolean.TRUE");
                }
            }

            private boolean setInformationDisplay(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    throw new FieldOptionException("optionValue cannot be null");
                }
                if (!(optionValue instanceof Short)) {
                    throw new FieldOptionException("optionValue must be a short");
                }
                short newVal = (Short) optionValue;
                short currentVal = informationDisplay;
                switch (newVal) {
                    case FieldOptionsConstants.INLINE_DISPLAY:
                    case FieldOptionsConstants.BLOCK_DISPLAY:
                        if (newVal != currentVal) {
                            informationDisplay = ((Short) optionValue);
                            return true;
                        } else {
                            return false;
                        }
                    default:
                        throw new FieldOptionException("unknown informationDisplay value = " + newVal);
                }
            }

            private boolean setDefaultSphereKey(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (sphereKey == null) {
                        return false;
                    } else {
                        sphereKey = null;
                        return true;
                    }
                } else {
                    if (!(optionValue instanceof SubsetKey)) {
                        throw new FieldOptionException("optionValue must be a SubsetKey instance");
                    }
                    SubsetKey subsetKey = (SubsetKey) optionValue;
                    if (!subsetKey.isSphereSubset()) {
                        throw new FieldOptionException("optionValue must be a sphere SubsetKey");
                    }
                    if ((sphereKey != null) && (sphereKey.equals(subsetKey))) {
                        return false;
                    }
                    sphereKey = subsetKey;
                    return true;
                }
            }

            private boolean setCurrencies(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (currencies == null) {
                        return false;
                    } else {
                        currencies = null;
                        return true;
                    }
                }
                Set<ExtendedCurrency> currencySet = new LinkedHashSet<ExtendedCurrency>();
                if (optionValue instanceof ExtendedCurrency[]) {
                    ExtendedCurrency[] currencyArray = (ExtendedCurrency[]) optionValue;
                    for (ExtendedCurrency currency : currencyArray) {
                        currencySet.add(currency);
                    }
                } else if (optionValue instanceof Currencies) {
                    currencySet.addAll((Currencies) optionValue);
                } else {
                    throw new FieldOptionException("optionValue must be an instance of ExtendedCurrency[] or Currencies");
                }
                if (currencySet.isEmpty()) {
                    if (currencies == null) {
                        return false;
                    } else {
                        currencies = null;
                        return true;
                    }
                }
                if ((currencies != null) && (CurrenciesUtils.areEquals(currencies, currencySet))) {
                    return false;
                }
                currencies = CurrenciesUtils.fromCollection(currencySet);
                return true;
            }

            private boolean setAddressFieldList(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (addressFieldList == null) {
                        return false;
                    } else {
                        addressFieldList = null;
                        return true;
                    }
                }
                FieldKey[] newArray = null;
                if (optionValue instanceof FieldKey[]) {
                    newArray = (FieldKey[]) optionValue;
                } else if (optionValue instanceof List) {
                    newArray = CorpusMetadataUtils.toFieldKeyArray((List<CorpusField>) optionValue);
                } else {
                    throw new FieldOptionException("optionValue must be an instance of CorpusField[] or CorpusFieldList");
                }
                List<CorpusField> corpusFieldList = new ArrayList<CorpusField>();
                for (FieldKey newFieldKey : newArray) {
                    CorpusField corpusField = CorpusMetadataImpl.this.getCorpusField(newFieldKey);
                    if (corpusField != null) {
                        corpusFieldList.add(corpusField);
                    }
                }
                if (corpusFieldList.isEmpty()) {
                    if (addressFieldList == null) {
                        return false;
                    } else {
                        addressFieldList = null;
                        return true;
                    }
                }
                if ((addressFieldList != null) && (CorpusMetadataUtils.areEquals(addressFieldList, corpusFieldList))) {
                    return false;
                }
                addressFieldList = CorpusMetadataUtils.wrap(corpusFieldList.toArray(new CorpusField[corpusFieldList.size()]));
                return true;
            }

            private boolean setLangScope(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    throw new FieldOptionException("optionValue cannot be null");
                }
                if (!(optionValue instanceof Short)) {
                    throw new FieldOptionException("optionValue must be a short");
                }
                short newVal = (Short) optionValue;
                short currentVal = langScope;
                switch (newVal) {
                    case FieldOptionsConstants.ALL_SCOPE:
                    case FieldOptionsConstants.CONFIG_SCOPE:
                    case FieldOptionsConstants.LIST_SCOPE:
                        if (newVal != currentVal) {
                            langScope = ((Short) optionValue);
                            return true;
                        } else {
                            return false;
                        }
                    default:
                        throw new FieldOptionException("unknown informationDisplay value = " + newVal);
                }
            }

            private boolean setLangs(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (langs == null) {
                        return false;
                    } else {
                        langs = null;
                        return true;
                    }
                }
                Set<Lang> langSet = new LinkedHashSet<Lang>();
                if (optionValue instanceof Lang[]) {
                    Lang[] langArray = (Lang[]) optionValue;
                    for (Lang lang : langArray) {
                        langSet.add(lang);
                    }
                } else if (optionValue instanceof Langs) {
                    langSet.addAll((Langs) optionValue);
                } else {
                    throw new FieldOptionException("optionValue must be an instance of Lang[] or Langs");
                }
                if (langSet.isEmpty()) {
                    if (langs == null) {
                        return false;
                    } else {
                        langs = null;
                        return true;
                    }
                }
                if ((langs != null) && (LangsUtils.areEquals(langs, langSet))) {
                    return false;
                }
                langs = LangsUtils.fromCollection(langSet);
                return true;
            }

        }

    }

    private synchronized CorpusField innerCreateCorpusField(FieldKey fieldKey, short ficheItemType) throws ExistingFieldKeyException {
        if (corpusFieldMap.containsKey(fieldKey)) {
            throw new ExistingFieldKeyException();
        }
        if (!fieldKey.isSpecialFieldKey()) {
            if (!StringUtils.isTechnicalName(fieldKey.getFieldName(), false)) {
                throw new IllegalArgumentException("fieldKey.getFieldName() is not valid : " + fieldKey.getFieldName());
            }
        }
        CorpusFieldImpl corpusField;
        if ((fieldKey.equals(FieldKey.ID)) || (fieldKey.equals(FieldKey.TITRE))) {
            corpusField = new CorpusFieldImpl(fieldKey);
        } else if (fieldKey.ownsToCorpsdefiche()) {
            corpusField = new CorpusFieldImpl(fieldKey);
        } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
            corpusField = new CorpusFieldImpl(fieldKey, CorpusField.PERSONNE_FIELD);
        } else if (fieldKey.equals(FieldKey.LANG)) {
            corpusField = new CorpusFieldImpl(fieldKey, CorpusField.LANGUE_FIELD);
        } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            corpusField = new CorpusFieldImpl(fieldKey, CorpusField.PARA_FIELD);
        } else {
            CorpusMetadataUtils.checkFicheItemType(ficheItemType);
            corpusField = new CorpusFieldImpl(fieldKey, ficheItemType);
        }
        if (fieldKey.isSectionFieldKey()) {
            sectionFieldList.add(corpusField);
        } else if (fieldKey.isProprieteFieldKey()) {
            proprieteFieldList.add(corpusField);
        } else if (fieldKey.isInformationFieldKey()) {
            informationFieldList.add(corpusField);
        }
        corpusFieldMap.put(fieldKey, corpusField);
        for (FieldGeneration.Entry entry : fieldGeneration.getEntryList()) {
            if (entry.getFieldKey().equals(fieldKey)) {
                corpusField.generated = true;
            }
        }
        clearCache();
        return corpusField;
    }

    private boolean innerPutLabel(CorpusField corpusField, Label label) {
        CorpusFieldImpl corpusFieldImpl = testCorpusField(corpusField);
        return corpusFieldImpl.putLabel(label);
    }

    private boolean innerRemoveLabel(CorpusField corpusField, Lang lang) {
        CorpusFieldImpl corpusFieldImpl = testCorpusField(corpusField);
        return corpusFieldImpl.removeLabel(lang);
    }

    private boolean innerSetFieldOption(CorpusField corpusField, String optionName, Object optionValue) throws FieldOptionException {
        CorpusFieldImpl corpusFieldImpl = testCorpusField(corpusField);
        return corpusFieldImpl.setOption(optionName, optionValue);
    }

    private synchronized void innerRemoveCorpusField(CorpusField corpusField) throws MandatoryFieldException {
        CorpusFieldImpl corpusFieldImpl = testCorpusField(corpusField);
        FieldKey fieldKey = corpusFieldImpl.getFieldKey();
        if (fieldKey.isMandatoryFieldKey()) {
            throw new MandatoryFieldException();
        }
        corpusFieldMap.remove(fieldKey);
        if (fieldKey.isProprieteFieldKey()) {
            proprieteFieldList.remove(corpusFieldImpl);
            if ((geolocalisationField != null) && (geolocalisationField.equals(corpusFieldImpl))) {
                geolocalisationField = null;
            }
        } else if (fieldKey.isSectionFieldKey()) {
            sectionFieldList.remove(corpusFieldImpl);
        } else if (fieldKey.isInformationFieldKey()) {
            informationFieldList.remove(corpusFieldImpl);
        }
        for (CorpusFieldImpl otherCorpusfield : proprieteFieldList) {
            if (otherCorpusfield.fieldOptions.addressFieldList != null) {
                otherCorpusfield.fieldOptions.addressFieldList = CorpusMetadataUtils.removeCorpusField(otherCorpusfield.fieldOptions.addressFieldList, corpusField);
            }
        }
        clearCache();
    }

    private boolean innerSetGeolocalisationField(CorpusField corpusField) {
        boolean done = false;
        if (corpusField == null) {
            if (geolocalisationField != null) {
                geolocalisationField = null;
                done = true;
            }
        } else {
            CorpusFieldImpl defcorpusfield = testCorpusField(corpusField);
            FieldKey fieldKey = defcorpusfield.getFieldKey();
            if (!fieldKey.isProprieteFieldKey()) {
                throw new IllegalArgumentException("not a Propriete : " + fieldKey.toString());
            }
            if (corpusField.getFicheItemType() != CorpusField.GEOPOINT_FIELD) {
                throw new IllegalArgumentException("not a GeoPoint Type : " + corpusField.getFicheItemType());
            }
            if (geolocalisationField == null) {
                geolocalisationField = defcorpusfield;
                done = true;
            } else if (!geolocalisationField.equals(defcorpusfield)) {
                geolocalisationField = defcorpusfield;
                done = true;
            }
        }
        return done;
    }

    private boolean innerSetFieldGeneration(FieldGeneration newFieldGeneration) {
        if (newFieldGeneration.getRawString().equals(fieldGeneration.getRawString())) {
            return false;
        }
        fieldGeneration = newFieldGeneration;
        for (CorpusFieldImpl corpusFieldImpl : corpusFieldMap.values()) {
            corpusFieldImpl.generated = false;
        }
        for (FieldGeneration.Entry fieldGenerationEntry : newFieldGeneration.getEntryList()) {
            CorpusFieldImpl corpusFieldImpl = corpusFieldMap.get(fieldGenerationEntry.getFieldKey());
            if (corpusFieldImpl != null) {
                corpusFieldImpl.generated = true;
            }
        }
        return true;
    }


    static class InitEditor implements CorpusMetadataEditor {

        private final CorpusMetadataImpl corpusMetadata;
        private final CorpusEditor corpusEditor;

        private InitEditor(CorpusMetadataImpl corpusMetadata, CorpusEditor corpusEditor) {
            this.corpusMetadata = corpusMetadata;
            this.corpusEditor = corpusEditor;
        }

        @Override
        public Metadata getMetadata() {
            return corpusMetadata;
        }

        @Override
        public FichothequeEditor getFichothequeEditor() {
            return corpusEditor.getFichothequeEditor();
        }

        @Override
        public CorpusEditor getCorpusEditor() {
            return corpusEditor;
        }

        @Override
        public CorpusField createCorpusField(FieldKey fieldKey, short ficheItemType) throws ExistingFieldKeyException {
            return corpusMetadata.innerCreateCorpusField(fieldKey, ficheItemType);
        }

        @Override
        public boolean putFieldLabel(CorpusField corpusField, Label label) {
            return corpusMetadata.innerPutLabel(corpusField, label);
        }

        @Override
        public boolean removeFieldLabel(CorpusField corpusField, Lang lang) {
            return corpusMetadata.innerRemoveLabel(corpusField, lang);
        }

        @Override
        public boolean putLabel(String name, Label label) {
            return corpusMetadata.innerPutLabel(name, label);
        }

        @Override
        public boolean removeLabel(String name, Lang lang) {
            return corpusMetadata.innerRemoveLabel(name, lang);
        }

        @Override
        public boolean setFieldOption(CorpusField corpusField, String optionName, Object optionValue) throws FieldOptionException {
            return corpusMetadata.innerSetFieldOption(corpusField, optionName, optionValue);
        }

        @Override
        public void removeCorpusField(CorpusField corpusField) throws MandatoryFieldException {
            corpusMetadata.innerRemoveCorpusField(corpusField);
        }

        @Override
        public boolean setGeolocalisationField(CorpusField corpusField) {
            return corpusMetadata.innerSetGeolocalisationField(corpusField);
        }

        @Override
        public boolean setFieldGeneration(FieldGeneration fieldGeneration) {
            return corpusMetadata.innerSetFieldGeneration(fieldGeneration);
        }

    }


    private class CorpusMetadataEditorImpl implements CorpusMetadataEditor {

        private final CorpusImpl.CorpusEditorImpl corpusEditor;

        private CorpusMetadataEditorImpl(CorpusImpl.CorpusEditorImpl corpusEditor) {
            this.corpusEditor = corpusEditor;
        }

        @Override
        public Metadata getMetadata() {
            return CorpusMetadataImpl.this;
        }

        @Override
        public FichothequeEditor getFichothequeEditor() {
            return corpusEditor.getFichothequeEditor();
        }

        @Override
        public CorpusEditor getCorpusEditor() {
            return corpusEditor;
        }

        @Override
        public CorpusField createCorpusField(FieldKey fieldKey, short ficheItemType) throws ExistingFieldKeyException {
            CorpusField corpusField = innerCreateCorpusField(fieldKey, ficheItemType);
            fireChange();
            getListeners().fireCorpusFieldCreated(corpusEditor.getFichothequeEditor(), corpus, corpusField);
            return corpusField;
        }

        @Override
        public boolean putFieldLabel(CorpusField corpusField, Label label) {
            boolean done = innerPutLabel(corpusField, label);
            if (done) {
                fireChange();
            }
            return done;
        }

        @Override
        public boolean removeFieldLabel(CorpusField corpusField, Lang lang) {
            boolean done = innerRemoveLabel(corpusField, lang);
            if (done) {
                fireChange();
            }
            return done;
        }

        @Override
        public boolean putLabel(String name, Label label) {
            boolean done = innerPutLabel(name, label);
            if (done) {
                fireChange();
            }
            return done;
        }

        @Override
        public boolean removeLabel(String name, Lang lang) {
            boolean done = innerRemoveLabel(name, lang);
            if (done) {
                fireChange();
            }
            return done;
        }

        @Override
        public boolean setFieldOption(CorpusField corpusField, String optionName, Object optionValue) throws FieldOptionException {
            boolean done = innerSetFieldOption(corpusField, optionName, optionValue);
            if (done) {
                fireChange();
            }
            return done;
        }

        @Override
        public void removeCorpusField(CorpusField corpusField) throws MandatoryFieldException {
            FieldKey fieldKey = corpusField.getFieldKey();
            innerRemoveCorpusField(corpusField);
            fireChange();
            getListeners().fireCorpusFieldRemoved(corpusEditor.getFichothequeEditor(), corpus, fieldKey);
        }

        @Override
        public boolean setGeolocalisationField(CorpusField corpusField) {
            boolean done = innerSetGeolocalisationField(corpusField);
            if (done) {
                fireChange();
            }
            return done;
        }

        @Override
        public boolean setFieldGeneration(FieldGeneration fieldGeneration) {
            boolean done = innerSetFieldGeneration(fieldGeneration);
            if (done) {
                fireChange();
            }
            return done;
        }

        private void fireChange() {
            corpusEditor.setMetadataChange();
        }

    }

}
