/* FichothequeLib_Tools - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheQueryBuilder {

    private final Set<SubsetKey> corpusKeySet = new LinkedHashSet<SubsetKey>();
    private final List<MotcleQuery> indexationList = new ArrayList<MotcleQuery>();
    private final Set<FieldKey> fieldKeySet = new LinkedHashSet<FieldKey>();
    private final Set<SubsetKey> sphereKeySet = new LinkedHashSet<SubsetKey>();
    private final Set<String> redacteurGlobalIdSet = new LinkedHashSet<String>();
    private Ranges idRanges = null;
    private Condition fieldQueryContentCondition = null;
    private short fieldQueryScope = FieldQuery.FICHE_SCOPE;
    private String redacteurFilter = FicheQuery.ALL_REDACTEURFILTER;
    private short motcleSelectMode = ConditionsConstants.MODE_AND;
    private CroisementCondition croisementCondition = null;
    private boolean withRattachement = false;
    private PeriodCondition periodCondition = null;
    private boolean withCurrentCorpus = false;
    private String discardFilter = FicheQuery.ALL_DISCARDFILTER;
    private boolean withGeoloc = false;

    public FicheQueryBuilder() {
    }

    public FicheQueryBuilder(FicheQuery ficheQuery) {
        this.corpusKeySet.addAll(ficheQuery.getCorpusKeyList());
        this.idRanges = ficheQuery.getIdRanges();
        this.redacteurFilter = ficheQuery.getRedacteurFilter();
        this.redacteurGlobalIdSet.addAll(ficheQuery.getRedacteurGlobalIdList());
        this.sphereKeySet.addAll(ficheQuery.getSphereKeyList());
        this.croisementCondition = ficheQuery.getCroisementCondition();
        this.withRattachement = ficheQuery.isWithRattachement();
        this.periodCondition = ficheQuery.getPeriodCondition();
        this.withCurrentCorpus = ficheQuery.isWithCurrentCorpus();
        this.discardFilter = ficheQuery.getDiscardFilter();
        this.withGeoloc = ficheQuery.isWithGeoloc();
        FieldQuery fieldQuery = ficheQuery.getFieldQuery();
        if (fieldQuery != null) {
            this.fieldQueryContentCondition = fieldQuery.getContentCondition();
            this.fieldQueryScope = fieldQuery.getScope();
            this.fieldKeySet.addAll(fieldQuery.getFieldKeyList());
        }
        MotcleCondition motcleCondition = ficheQuery.getMotcleCondition();
        if (motcleCondition != null) {
            this.indexationList.addAll(motcleCondition.getMotcleQueryList());
            this.motcleSelectMode = motcleCondition.getSelectMode();
        }
    }

    public FicheQueryBuilder addCorpus(Object object) {
        if (object == null) {
            return this;
        }
        if (object instanceof Corpus) {
            corpusKeySet.add(((Corpus) object).getSubsetKey());
        } else if (object instanceof SubsetKey) {
            SubsetKey subsetKey = (SubsetKey) object;
            if (subsetKey.isCorpusSubset()) {
                corpusKeySet.add(subsetKey);
            }
        } else if (object instanceof Collection) {
            for (Object obj : ((Collection) object)) {
                if (obj != null) {
                    if (obj instanceof Corpus) {
                        corpusKeySet.add(((Corpus) obj).getSubsetKey());
                    } else if (obj instanceof SubsetKey) {
                        SubsetKey subsetKey = (SubsetKey) obj;
                        if (subsetKey.isCorpusSubset()) {
                            corpusKeySet.add(subsetKey);
                        }
                    }
                }
            }
        } else if (object instanceof SubsetKey[]) {
            for (SubsetKey subsetKey : ((SubsetKey[]) object)) {
                if ((subsetKey != null) && (subsetKey.isCorpusSubset())) {
                    corpusKeySet.add(subsetKey);
                }
            }
        }
        return this;
    }

    public FicheQueryBuilder clearCorpus() {
        corpusKeySet.clear();
        return this;
    }

    /**
     *
     * @throws IllegalArgumentException si rangeList n'est pas positif
     */
    public FicheQueryBuilder setIdRanges(Ranges ranges) {
        if ((ranges != null) && (!RangeUtils.isPositiveRangeList(ranges))) {
            throw new IllegalArgumentException("rangeList is not positive");
        }
        this.idRanges = ranges;
        return this;
    }

    public FicheQueryBuilder addIndexationMotcleQuery(MotcleQuery motcleQuery) {
        if (motcleQuery == null) {
            throw new IllegalArgumentException("motcleQuery is null");
        }
        indexationList.add(motcleQuery);
        return this;
    }

    public FicheQueryBuilder setMotcleSelectMode(short motcleSelectMode) {
        this.motcleSelectMode = motcleSelectMode;
        return this;
    }

    public FicheQueryBuilder setWithCurrentCorpus(boolean withCurrentCorpus) {
        this.withCurrentCorpus = withCurrentCorpus;
        return this;
    }

    public FicheQueryBuilder setFieldCondition(Condition fieldCondition, short fieldConditionScope, Collection<FieldKey> fieldKeys) {
        switch (fieldConditionScope) {
            case FieldQuery.TITRE_SCOPE:
            case FieldQuery.ENTETE_SCOPE:
            case FieldQuery.FICHE_SCOPE:
            case FieldQuery.SELECTION_SCOPE:
                this.fieldQueryScope = fieldConditionScope;
                break;
            default:
                throw new IllegalArgumentException("Unknown fieldConditionScope : " + fieldConditionScope);
        }
        this.fieldQueryContentCondition = fieldCondition;
        fieldKeySet.clear();
        if (fieldConditionScope == FieldQuery.SELECTION_SCOPE) {
            if (fieldKeys == null) {
                throw new IllegalArgumentException("fieldKeys is null");
            }
            if (fieldKeys.isEmpty()) {
                throw new IllegalArgumentException("fieldKeys is empty");
            }
            fieldKeySet.addAll(fieldKeys);
        }
        return this;
    }

    public FicheQueryBuilder setRedacteurFilter(String redacteurFilter) {
        if (redacteurFilter.equals(FicheQuery.SOME_REDACTEURFILTER)) {
            throw new IllegalArgumentException("redacteurFilter == FicheQuery.SOME_REDACTEURFILTER");
        }
        this.redacteurFilter = redacteurFilter;
        sphereKeySet.clear();
        redacteurGlobalIdSet.clear();
        return this;
    }

    public FicheQueryBuilder setRedacteurFilter(Collection<String> redacteurGlobalIds, Collection<SubsetKey> sphereKeys) {
        this.redacteurFilter = FicheQuery.SOME_REDACTEURFILTER;
        for (String redacteurGlobalId : redacteurGlobalIds) {
            if (redacteurGlobalId != null) {
                redacteurGlobalIdSet.add(redacteurGlobalId);
            }
        }
        for (SubsetKey sphereKey : sphereKeys) {
            if (sphereKey != null) {
                if (!sphereKey.isSphereSubset()) {
                    throw new IllegalArgumentException("!subsetKey.isCorpusSubset(): " + sphereKey.toString());
                }
                sphereKeySet.add(sphereKey);
            }
        }
        return this;
    }

    public FicheQueryBuilder setCroisementCondition(CroisementCondition croisementCondition) {
        this.croisementCondition = croisementCondition;
        return this;
    }

    public FicheQueryBuilder setWithRattachement(boolean withRattachement) {
        this.withRattachement = withRattachement;
        return this;
    }

    public FicheQueryBuilder setPeriodCondition(PeriodCondition periodCondition) {
        this.periodCondition = periodCondition;
        return this;
    }

    public FicheQueryBuilder setDiscardFilter(String discardFilter) {
        this.discardFilter = SelectionUtils.checkDiscardFilter(discardFilter);
        return this;
    }

    public FicheQueryBuilder setWithGeoloc(boolean withGeoloc) {
        this.withGeoloc = withGeoloc;
        return this;
    }

    public FicheQuery toFicheQuery() {
        List<SubsetKey> corpusKeyList = FichothequeUtils.toList(corpusKeySet);
        List<SubsetKey> sphereKeyList = FichothequeUtils.toList(sphereKeySet);
        List<String> redacteurGlobalIdList = StringUtils.toList(redacteurGlobalIdSet);
        FieldQuery fieldQuery = null;
        if (fieldQueryContentCondition != null) {
            List<FieldKey> fieldKeyList;
            if (fieldKeySet.isEmpty()) {
                fieldKeyList = CorpusMetadataUtils.EMPTY_FIELDKEYLIST;
            } else {
                fieldKeyList = CorpusMetadataUtils.wrap(fieldKeySet.toArray(new FieldKey[fieldKeySet.size()]));
            }
            fieldQuery = new InternalFieldQuery(fieldQueryScope, fieldKeyList, fieldQueryContentCondition);
        }
        MotcleCondition motcleCondition;
        if (indexationList.isEmpty()) {
            motcleCondition = null;
        } else {
            motcleCondition = SelectionUtils.toMotcleCondition(SelectionUtils.wrap(indexationList.toArray(new MotcleQuery[indexationList.size()])), motcleSelectMode);
        }
        return new InternalFicheQuery(corpusKeyList, idRanges, motcleCondition,
                fieldQuery, redacteurFilter, redacteurGlobalIdList,
                sphereKeyList, croisementCondition, withRattachement, periodCondition, withCurrentCorpus, discardFilter, withGeoloc);
    }

    public static FicheQueryBuilder init() {
        return new FicheQueryBuilder();
    }


    private static class InternalFicheQuery implements FicheQuery {

        private final List<SubsetKey> corpusKeyList;
        private final Ranges idRanges;
        private final MotcleCondition motcleCondition;
        private final FieldQuery fieldQuery;
        private final String redacteurFilter;
        private final List<String> redacteurGlobalIdList;
        private final List<SubsetKey> sphereKeyList;
        private final CroisementCondition croisementCondition;
        private final boolean withRattachement;
        private final PeriodCondition periodCondition;
        private final boolean withCurrentCorpus;
        private final String discardFilter;
        private final boolean withGeoloc;

        private InternalFicheQuery(List<SubsetKey> corpusKeyList, Ranges idRanges, MotcleCondition motcleCondition,
                FieldQuery fieldQuery, String redacteurFilter, List<String> redacteurGlobalIdList,
                List<SubsetKey> sphereKeyList, CroisementCondition croisementCondition, boolean withRattachement, PeriodCondition periodCondition, boolean withCurrentCorpus, String discardFilter, boolean withGeoloc) {
            this.corpusKeyList = corpusKeyList;
            this.idRanges = idRanges;
            this.motcleCondition = motcleCondition;
            this.fieldQuery = fieldQuery;
            this.redacteurFilter = redacteurFilter;
            this.redacteurGlobalIdList = redacteurGlobalIdList;
            this.sphereKeyList = sphereKeyList;
            this.croisementCondition = croisementCondition;
            this.withRattachement = withRattachement;
            this.periodCondition = periodCondition;
            this.withCurrentCorpus = withCurrentCorpus;
            this.discardFilter = discardFilter;
            this.withGeoloc = withGeoloc;
        }

        @Override
        public List<SubsetKey> getCorpusKeyList() {
            return corpusKeyList;
        }

        @Override
        public Ranges getIdRanges() {
            return idRanges;
        }

        @Override
        public MotcleCondition getMotcleCondition() {
            return motcleCondition;
        }

        @Override
        public FieldQuery getFieldQuery() {
            return fieldQuery;
        }

        @Override
        public List<String> getRedacteurGlobalIdList() {
            return redacteurGlobalIdList;
        }

        @Override
        public List<SubsetKey> getSphereKeyList() {
            return sphereKeyList;
        }

        @Override
        public String getRedacteurFilter() {
            return redacteurFilter;
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return croisementCondition;
        }

        @Override
        public boolean isWithRattachement() {
            return withRattachement;
        }


        @Override
        public PeriodCondition getPeriodCondition() {
            return periodCondition;
        }

        @Override
        public boolean isWithCurrentCorpus() {
            return withCurrentCorpus;
        }

        @Override
        public String getDiscardFilter() {
            return discardFilter;
        }

        @Override
        public boolean isWithGeoloc() {
            return withGeoloc;
        }

    }


    private static class InternalFieldQuery implements FieldQuery {

        private final short scope;
        private final List<FieldKey> fieldKeyList;
        private final Condition contentCondition;

        private InternalFieldQuery(short scope, List<FieldKey> fieldKeyList, Condition contentCondition) {
            this.scope = scope;
            this.fieldKeyList = fieldKeyList;
            this.contentCondition = contentCondition;
        }

        @Override
        public short getScope() {
            return scope;
        }

        @Override
        public List<FieldKey> getFieldKeyList() {
            return fieldKeyList;
        }

        @Override
        public Condition getContentCondition() {
            return contentCondition;
        }

    }

}
