/* FichothequeLib_Tools - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.SubsetKey;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.primitives.Ranges;


/**
 *
 * @author Vincent Calame
 */
public class MotcleQueryBuilder {

    private final Set<SubsetKey> thesaurusKeySet = new LinkedHashSet<SubsetKey>();
    private final List<FicheQuery> indexationList = new ArrayList<FicheQuery>();
    private Ranges idRanges = null;
    private Ranges levelRanges = null;
    private Condition contentCondition = null;
    private short contentConditionScope = MotcleQuery.WITH_IDALPHA_SCOPE;
    private short ficheSelectMode = ConditionsConstants.MODE_AND;
    private CroisementCondition croisementCondition = null;
    private boolean includeSatellites = false;


    public MotcleQueryBuilder() {

    }

    public MotcleQueryBuilder(MotcleQuery motcleQuery) {
        this.idRanges = motcleQuery.getIdRanges();
        this.levelRanges = motcleQuery.getLevelRanges();
        this.thesaurusKeySet.addAll(motcleQuery.getThesaurusKeyList());
        this.contentCondition = motcleQuery.getContentCondition();
        this.contentConditionScope = motcleQuery.getContentConditionScope();
        this.croisementCondition = motcleQuery.getCroisementCondition();
        this.includeSatellites = motcleQuery.includeSatellites();
        FicheCondition ficheCondition = motcleQuery.getFicheCondition();
        if (ficheCondition != null) {
            this.indexationList.addAll(ficheCondition.getFicheQueryList());
            this.ficheSelectMode = ficheCondition.getSelectMode();
        }
    }

    public MotcleQueryBuilder setIdRangeList(Ranges idRanges) {
        if ((idRanges == null) || (idRanges.isEmpty())) {
            this.idRanges = null;
        } else {
            if (!RangeUtils.isPositiveRangeList(idRanges)) {
                throw new IllegalArgumentException("rangeList is not positive");
            }
            this.idRanges = idRanges;
        }
        return this;
    }

    public MotcleQueryBuilder setLevelRangeList(Ranges levelRanges) {
        if ((levelRanges == null) || (levelRanges.isEmpty())) {
            this.levelRanges = null;
        } else {
            this.levelRanges = levelRanges;
        }
        return this;
    }

    public MotcleQueryBuilder addThesaurus(Object object) {
        if (object == null) {
            return this;
        }
        if (object instanceof Thesaurus) {
            thesaurusKeySet.add(((Thesaurus) object).getSubsetKey());
        } else if (object instanceof SubsetKey) {
            SubsetKey subsetKey = (SubsetKey) object;
            if (subsetKey.isThesaurusSubset()) {
                thesaurusKeySet.add(subsetKey);
            }
        } else if (object instanceof Collection) {
            for (Object obj : ((Collection) object)) {
                if (obj != null) {
                    if (obj instanceof Thesaurus) {
                        thesaurusKeySet.add(((Thesaurus) obj).getSubsetKey());
                    } else if (obj instanceof SubsetKey) {
                        SubsetKey subsetKey = (SubsetKey) obj;
                        if (subsetKey.isThesaurusSubset()) {
                            thesaurusKeySet.add(subsetKey);
                        }
                    }
                }
            }
        } else if (object instanceof SubsetKey[]) {
            for (SubsetKey subsetKey : ((SubsetKey[]) object)) {
                if ((subsetKey != null) && (subsetKey.isThesaurusSubset())) {
                    thesaurusKeySet.add(subsetKey);
                }
            }
        }
        return this;
    }

    public MotcleQueryBuilder setContentCondition(Condition contentCondition, short contentConditionScope) {
        this.contentCondition = contentCondition;
        this.contentConditionScope = contentConditionScope;
        return this;
    }

    public MotcleQueryBuilder setContentCondition(Condition contentCondition) {
        this.contentCondition = contentCondition;
        return this;
    }

    public MotcleQueryBuilder setContentConditionScope(short contentConditionScope) {
        this.contentConditionScope = contentConditionScope;
        return this;
    }

    public MotcleQueryBuilder addIndexationFicheQuery(FicheQuery ficheQuery) {
        if (ficheQuery == null) {
            throw new IllegalArgumentException("ficheQuery is null");
        }
        indexationList.add(ficheQuery);
        return this;
    }

    public MotcleQueryBuilder setFicheSelectMode(short ficheSelectMode) {
        this.ficheSelectMode = ficheSelectMode;
        return this;
    }

    public MotcleQueryBuilder setCroisementCondition(CroisementCondition croisementCondition) {
        this.croisementCondition = croisementCondition;
        return this;
    }

    public MotcleQueryBuilder setIncludeSatellites(boolean includeSatellites) {
        this.includeSatellites = includeSatellites;
        return this;
    }

    public MotcleQuery toMotcleQuery() {
        List<SubsetKey> thesaurusKeyList = FichothequeUtils.toList(thesaurusKeySet);
        FicheCondition ficheCondition;
        if (indexationList.isEmpty()) {
            ficheCondition = null;
        } else {
            ficheCondition = SelectionUtils.toFicheCondition(SelectionUtils.wrap(indexationList.toArray(new FicheQuery[indexationList.size()])), ficheSelectMode);
        }
        return new InternalMotcleQuery(idRanges, levelRanges, thesaurusKeyList, contentCondition, contentConditionScope, ficheCondition, croisementCondition, includeSatellites);
    }

    public static MotcleQueryBuilder init() {
        return new MotcleQueryBuilder();
    }


    private static class InternalMotcleQuery implements MotcleQuery {

        private final Ranges idRanges;
        private final Ranges levelRanges;
        private final List<SubsetKey> thesaurusKeyList;
        private final Condition contentCondition;
        private final short contentConditionScope;
        private final boolean includeSatellites;
        private final FicheCondition ficheCondition;
        private final CroisementCondition croisementCondition;

        private InternalMotcleQuery(Ranges idRanges, Ranges levelRanges,
                List<SubsetKey> subsetKeyList, Condition contentCondition, short contentConditionScope,
                FicheCondition ficheCondition, CroisementCondition croisementCondition, boolean includeSatellites) {
            this.idRanges = idRanges;
            this.levelRanges = levelRanges;
            this.thesaurusKeyList = subsetKeyList;
            this.contentCondition = contentCondition;
            this.contentConditionScope = contentConditionScope;
            this.ficheCondition = ficheCondition;
            this.croisementCondition = croisementCondition;
            this.includeSatellites = includeSatellites;
        }

        @Override
        public Ranges getIdRanges() {
            return idRanges;
        }

        @Override
        public Ranges getLevelRanges() {
            return levelRanges;
        }

        @Override
        public List<SubsetKey> getThesaurusKeyList() {
            return thesaurusKeyList;
        }

        @Override
        public Condition getContentCondition() {
            return contentCondition;
        }

        @Override
        public short getContentConditionScope() {
            return contentConditionScope;
        }

        @Override
        public FicheCondition getFicheCondition() {
            return ficheCondition;
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return croisementCondition;
        }

        @Override
        public boolean includeSatellites() {
            return includeSatellites;
        }

    }

}
