/* BdfServer - Copyright (c) 2017-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import java.util.Collection;
import java.util.Comparator;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.function.Predicate;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.selection.DocumentSelector;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.RedacteurSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.Comparators;
import net.fichotheque.utils.CroisementUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionEngines {

    private SelectionEngines() {
    }

    public static Collection<FicheMeta> run(SelectionContext selectionContext, FicheSelector ficheSelector) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        TreeSet<FicheMeta> treeset = new TreeSet<FicheMeta>(Comparators.FICHEID);
        for (Corpus corpus : ficheSelector.getCorpusList()) {
            if (subsetAccessPredicate.test(corpus)) {
                for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                    if (ficheSelector.test(ficheMeta)) {
                        treeset.add(ficheMeta);
                    }
                }
            }
        }
        return treeset;
    }

    public static Collection<Motcle> run(SelectionContext selectionContext, SubsetKey shouldNotCroisementSubsetKey, MotcleSelector motcleSelector) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        TreeSet<Motcle> treeSet = new TreeSet<Motcle>(new MotcleComparator(selectionContext.getWorkingLang()));
        for (Thesaurus thesaurus : motcleSelector.getThesaurusList()) {
            if (subsetAccessPredicate.test(thesaurus)) {
                for (Motcle motcle : thesaurus.getMotcleList()) {
                    if ((!shouldNotTest(motcle, shouldNotCroisementSubsetKey)) && (motcleSelector.test(motcle))) {
                        treeSet.add(motcle);
                    }
                }
            }
        }
        return treeSet;
    }

    public static Collection<Document> run(SelectionContext selectionContext, DocumentSelector documentSelector) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        TreeMap<String, Document> documentMap = new TreeMap<String, Document>();
        for (Addenda addenda : documentSelector.getAddendaList()) {
            if (subsetAccessPredicate.test(addenda)) {
                for (Document document : addenda.getDocumentList()) {
                    if (documentSelector.test(document)) {
                        documentMap.put(document.getBasename(), document);
                    }
                }
            }
        }
        return documentMap.values();
    }

    public static Collection<Redacteur> run(SelectionContext selectionContext, RedacteurSelector redacteurSelector) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        TreeSet<Redacteur> redacSet = new TreeSet<Redacteur>(new RedacteurComparator());
        for (Sphere sphere : redacteurSelector.getSphereList()) {
            if (subsetAccessPredicate.test(sphere)) {
                for (Redacteur redacteur : sphere.getRedacteurList()) {
                    if (!redacteur.isActive()) {
                        continue;
                    }
                    if (redacteurSelector.test(redacteur)) {
                        redacSet.add(redacteur);
                    }
                }
            }
        }
        return redacSet;
    }

    private static boolean shouldNotTest(Motcle motcle, SubsetKey shouldNotCroisementSubsetKey) {
        if (shouldNotCroisementSubsetKey == null) {
            return false;
        }
        return CroisementUtils.isShouldNotCroisement(motcle, shouldNotCroisementSubsetKey);
    }


    private static class MotcleComparator implements Comparator<Motcle> {

        private final Lang lang;

        public MotcleComparator(Lang lang) {
            this.lang = lang;
        }

        @Override
        public int compare(Motcle mc1, Motcle mc2) {
            String lib1 = toComparableString(mc1, lang);
            String lib2 = toComparableString(mc2, lang);
            int result;
            result = lib1.compareTo(lib2);
            if (result != 0) {
                return result;
            }
            result = mc1.getThesaurus().getSubsetKey().compareTo(mc2.getThesaurus().getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = mc1.getId();
            int id2 = mc2.getId();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

    private static String toComparableString(Motcle motcle, Lang lang) {
        String idalpha = motcle.getIdalpha();
        Label label = motcle.getLabels().getLabel(lang);
        if (idalpha != null) {
            if (label != null) {
                return idalpha + " – " + label.getLabelString();
            } else {
                return idalpha;
            }
        } else {
            if (label != null) {
                return label.getLabelString();
            } else {
                return "";
            }
        }
    }


    private static class RedacteurComparator implements Comparator<Redacteur> {

        private RedacteurComparator() {
        }

        @Override
        public int compare(Redacteur r1, Redacteur r2) {
            String id1 = r1.getLogin();
            String id2 = r2.getLogin();
            int result = id1.compareTo(id2);
            if (result != 0) {
                return result;
            }
            String nc1 = r1.getPersonCore().toDirectoryStyle(true);
            String nc2 = r2.getPersonCore().toDirectoryStyle(true);
            result = nc1.compareTo(nc2);
            if (result != 0) {
                return result;
            }
            return r1.getSphere().getSubsetKey().compareTo(r2.getSphere().getSubsetKey());
        }

    }

}
